/******************************************************************************
* C Include: Header                                                           *
*                                                                             *
* File:                                                                       *
*    header.h                                                                 *
*                                                                             *
* Description:                                                                *
*    Include file for the functions used to process header information for a  *
*    plate.                                                                   *
*                                                                             *
* History:                                                                    *
*    Created    : unknown   by R. White.                                      *
*    Delivered  : unknown   by R. White.                                      *
*                                                                             *
*    Updated    : 27-MAY-93 by J. Doggett,                                    *
*       Added symbolic constant HEADER_EXTENSION to change extensions for     *
*          header files.                                                      *
*    Updated    : 13-AUG-93 by J. Doggett,                                    *
*       Adopted into software used to facilitate the accessing of data from   *
*          the CD-ROM set of the Digitized Sky Survey.                        *
*    Updated    : 07-DEC-93 by J. Doggett,                                    *
*       Corrected misspelled prototype name for fillcopyright function.       *
*    Updated    : 14-DEC-93 by J. Doggett,                                    *
*       Added Check_Header_Plate_Id function prototype from hdrlist.h         *
*       Added __HEADER__ constant.                                            *
*       Added argument type comments to function prototypes                   *
*       Change prototype for fillhdr function.                                
*    Updated    : 24-JAN-94 by J. Doggett,                                    
*       Changed environment variables from CMP_xxxx to DSS_xxxx for Digitized 
*          Sky Survey.                                                        
*    Updated    : 17-FEB-94 by J. Doggett,                                    
*       Removed fillcopyright function prototype.                             
*    Redelivered: 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 20-JUL-94 by J. Doggett,                                    
*       Updated gethdr prototype to reflect change in its argument list.      
*    Redelivered: 30-NOV-94 by J. Doggett,                                    
*       GetImage Version 1.1.                                                 
*                                                                             
*    Updated    : 07-APR-95 by J. Doggett,                                    
*       Increased hlines limit from 200 to 1000.                              
*       Added copyright statements keywords to header.                        
*    Redelivered: 18-AUG-95 by J. Doggett,                                    
*       GetImage Version 1.2.                                                 
*                                                                             
* Copyright (c) 1993-1995, Association of Universities for Research in        
* Astronomy, Inc.  All Rights Reserved.                                       
* Produced under National Aeronautics and Space Administration grant          
* NAG W-2166.                                                                 
******************************************************************************/
#ifndef __HEADER__
#define __HEADER__

#include <ctype.h>
#include <stdio.h>
#include <string.h>

#include "generic.h"

/*******************
* Header constants *
*******************/

#define HEADER_DIRECTORY        "DSS_HDRS"

/*****************************
* Header include file page 2 *
*****************************/

#define HEADER_EXTENSION        "DSS_EXTN"

/****************
* Header macros *
****************/
#define ival(skey,v)  if (strcmp(keyword,skey)==0) { v=atoi(value); continue; }
#define fval(skey,v)  if (strcmp(keyword,skey)==0) { v=atof(value); continue; }
#define cvalq(skey,v) if (strcmp(keyword,skey)==0) {                           \
                      strncpy(v,remquotes(value),sizeof(v)-1);                 \
                      v[sizeof(v)-1]=(char)NULL;                               \
                      continue; }
#define cvalt(skey,v) if (strcmp(keyword,skey)==0) {                           \
                      strncpy(v,strtrim(remquotes(value)),sizeof(v)-1);        \
                      v[sizeof(v)-1]=(char)NULL;                               \
                      continue; }

/****************************
* Header derived data types *
****************************/
struct Header_Info{
    /*
     * plate directory information
     */
    char plate_root[80];    /* directory root for plates               */
    char header_dir[80];    /* directory root for header files         */
    char compression[7];    /* compression to use (low, medium, high)  */
    char plate_name[7];     /* name of this plate                      */
    /*
     * plate id, region identification
     */
    char plate_id[5], region_no[7];
    /*
     * size of section and corner position
     */
    int section_x_length, section_y_length;
    float section_x_corner, section_y_corner;
    /*
     * target position
     */
    int ra_h, ra_m, dec_d, dec_m;
    float ra_s, dec_s;
    char dec_sign;
    float x, y;
    /*
     * plate solution parameters
     */
    double plt_scale, x_pixel_size, y_pixel_size,
        plt_center_vec[3], ppo_coeff[6],
        amd_x_coeff[20], amd_y_coeff[20],
        plt_center_ra, plt_center_dec;
    char amd_flag, ppo_flag, plate_data_flag,
        special_plate_flag;
    /*
     * miscellaneous information from header keywords
     */
    int plt_grade, emulsion_code;
    float scaling, mag_limit, plt_date;
    char object_name[19], exposure_time[19], seeing[4], origin[19],
        plate_label[7];


/*****************************
* Header include file page 3 *
*****************************/

    /*
     * copy right information
     */
    int numBriefCopyRightStatements, numFullCopyRightStatements;
    char ** briefCopyRightStatements;
    char ** fullCopyRightStatements;
    /*
     * header lines
     */
    char hlines[200][81];
    int nhlines;
};
typedef struct Header_Info * HEADER;

/*******************************
* Header functions' prototypes *
*******************************/
int           Check_Header_Plate_Id(/* HEADER      Header          ,
                                       STRING      Plate_Id         */);

       void   Delete_Header        (/* HEADER      Header           */);

extern void   fillcopyright        (/* STRING      Source          ,
                                       HEADER      h                */);

extern void   fillhdr              (/* HEADER      h               ,
                                       char       *object          ,
                                       int         nx              ,
                                       int         ny              ,
                                       int         x0              ,
                                       int         y0              ,
                                       int         datamin         ,
                                       int         datamax          */);

extern void   fillpos              (/* HEADER      h               ,
                                       int         rah             ,
                                       int         ram             ,
                                       double      ras             ,
                                       char        decsign         ,
                                       int         decd            ,
                                       int         decm            ,
                                       double      decs            ,
                                       double      x               ,
                                       double      y                */);

extern HEADER gethdr               (/* STRING      Header_File_Name */);

       STRING Make_Header_File_Name(/* STRING      Directory       ,
                                       STRING      Plate_Name      ,
                                       STRING      Header_Extension */);

extern void   readhdr              (/* FILE       *infile          ,
                                       HEADER      h                */);

#endif
