/*++++++++++++++
.IDENTIFICATION skio.c
.LANGUAGE       C
.AUTHOR         Francois Ochsenbein [CDS]
.ENVIRONMENT    
.KEYWORDS       
.VERSION  1.0   04-Aug-1992
.VERSION  1.1   20-Aug-1992: Added sk_getl, sk_putl
.VERSION  1.2   12-Dec-1992: Added sk_setb
.VERSION  2.0   10-May-1993:
.COMMENTS       Read/Write on Sockets
\begin{TeX}

\end{TeX} 
- ---------------*/

#ifndef NSOCKS
#define NSOCKS 16
#endif

#if unix
#include <sys/types.h>  /* ntohl */
#include <netinet/in.h> /* Byte order */
#else
#include <types.h>
#include <in.h>
#endif

typedef void (*FCT)();
extern void perror();
static FCT iolog = perror;
static int sksize = 0;		/* Individual size of sockets */

#ifdef DEBUG
#include <stdio.h>
#define LOGFILE 	"/tmp/skio.log"
static int logs = 1;
static FILE *logfile;

static int logb(fct, buf, len)
/*++++++++++++++++
.PURPOSE  Log what's comes in
.RETURNS  0
- -----------------*/
	char *fct, *buf; int len;
{ int lmax, i;  
  if (!logfile) logfile = fopen(LOGFILE, "w");
  if (!logfile) logfile = stderr;
  fprintf(logfile, "%6s: %d bytes <", fct, len);
  lmax = len; if (lmax > 20) lmax = 20;
  for (i=lmax; --i >= 0; buf++) {
	if (isprint(*buf)) fprintf(logfile, "%c", *buf);
	else if ((*buf >= 0) && (*buf < 26)) 
		fprintf(logfile, "^%c", *buf+('A'-1));
	else fprintf(logfile, "^.");
  }
  fprintf(logfile, ">\n"); fflush(logfile);
  return(0);
}

static int logl(fct, val)
/*++++++++++++++++
.PURPOSE  Log what's comes in
.RETURNS  0
- -----------------*/
	char *fct; int val;
{ 
  if (!logfile) logfile = fopen(LOGFILE, "w");
  fprintf(logfile, "%6s= %d\n", fct, val);
  fflush(logfile);
  return(0);
}
#endif


static int err3(ssp, plug, len)
/*++++++++++++++++
.PURPOSE  Error report
.RETURNS  0
- -----------------*/
  char *ssp;	/* IN: Subroutine name */
  int  plug;	/* IN: Socket number */
  int len;	/* IN: Length variable */
{
	char buffer[128];

  sprintf(buffer, "****%s plug#%d for %d bytes", ssp, plug, len);
  (*iolog)(buffer);
  return(0);
}

int sk_setb(block)
/*++++++++++++++++
.PURPOSE  Set the size of blocks for socket transfer
.RETURNS  The previous value of the socket size
.REMARKS  A value < 1 does not change the socket blocksize.
- -----------------*/
   int block;	/* IN: Size of socket blocks in bytes (< 511) of blocks */
{
   int ob;

  ob = sksize; 
  if (block > 0) {
	sksize = block;
	if (sksize < 511) sksize *= 512;
  }
  return(ob);
}

FCT sk_iolog(f)
/*++++++++++++++++
.PURPOSE  Set the iolog function
.RETURNS  The previous iolog function
.REMARKS  Default is perror.
- -----------------*/
  FCT f;	/* IN: New iolog function */
{
	FCT o;

  o = iolog;
  iolog = f ? f : perror;
  return(o);
}

int sk_read(fno, buf, len)
/*++++++++++++++++
.PURPOSE  Read up to a specified number of bytes on a socket.
.RETURNS  Number of bytes read
.REMARKS  No attempt to fill the buffer.
- -----------------*/
	int fno;	/* IN: File Descriptor */
	char *buf;	/* OUT: Buffer filled  */
	int  len;	/* IN: Max len to read */
{
	int i;

  i = read(fno, buf, len);
  if (i < 0) err3("sk_read", fno, len); 
#ifdef DEBUG
	logb("sk_read", buf, i);
#endif
  return (i);
}

int sk_get(fno, buf, len)
/*++++++++++++++++
.PURPOSE  Read the specified number of bytes on a socket.
.RETURNS  Number of bytes read. Is normally equal to len.
- -----------------*/
	int fno;	/* IN: File Descriptor */
	char *buf;	/* OUT: Buffer filled  */
	int  len;	/* IN: Max len to read */
{
	char *p, *pe;
	int i;

  for (p = buf, pe = p+len; p < pe; p += i) {
  	i = read(fno, p, pe-p);
  	if (i < 0) err3("sk_get", fno, len); 
	if (i <= 0) break;
  }
#ifdef DEBUG
	if(logs) logb("sk_get", buf, p-buf);
#endif
  return (p - buf);
}

int sk_gets(fno, buf, len)
/*++++++++++++++++
.PURPOSE  Read a line (terminated by a newline) on a socket
.RETURNS  Number of bytes read, including the \n. 
.REMARKS  No NUL character appended.
- -----------------*/
	int fno;	/* IN: File Descriptor */
	char *buf;	/* OUT: Buffer filled  */
	int  len;	/* IN: Max len to read */
{
	char *p, *pe;
	int i;

  for (p = buf, pe = p+len; p < pe; ) {
  	i = read(fno, p, pe-p);
	if (i < 0) err3("sk_gets", fno, len);
	if (i <= 0) break;	/* End of File */
	p += i;
	if (p[-1] == '\n') break;
  }
#ifdef DEBUG
	logb("sk_gets", buf, p-buf);
#endif
  return (p - buf);
}

int sk_getl(fno)
/*++++++++++++++++
.PURPOSE  Read a 32-bit integer on a socket
.RETURNS  What's read; -1 when error
- -----------------*/
	int fno;        /* IN: opened socket */
{
	int stat;

#ifdef DEBUG
	logs = 0;
#endif
  if (sk_get(fno, &stat, sizeof(stat)) < 0) return(-1L);
#ifdef DEBUG
	logl("sk_getl", ntohl(stat));
	logs = 1;
#endif
  return(ntohl(stat));
}

/*============================================================================*/

int sk_write(fno, buf, len)
/*++++++++++++++++
.PURPOSE  Write a specified number of bytes on a socket.
.RETURNS  Number of bytes written, normally identical to len
.REMARKS  Several trials if necessary
- -----------------*/
	int fno;	/* IN: File Descriptor */
	char *buf;	/* IN: Buffer to send  */
	int  len;	/* IN: Max len to read */
{
	char *p, *pe;
	int i, b;

  for (p = buf, pe = p + len; p < pe; p += i) {
	b = pe-p;
	if (sksize && (b > sksize)) b = sksize;
	i = write(fno, p, b);
	if (i <= 0) { 
		if (i < 0) err3("sk_write", fno, len);
		break;
	}
  }
#ifdef DEBUG
	logb("sk_write", buf, p-buf);
#endif
  return (p - buf);
}

int sk_put(fno, buf)
/*++++++++++++++++
.PURPOSE  Write a text terminated by a NUL character
.RETURNS  Number of bytes written
.REMARKS  Several trials if necessary
- -----------------*/
	int fno;	/* IN: File Descriptor */
	char *buf;	/* IN: Buffer to send  */
{
  return(sk_write(fno, buf, strlen(buf)));
}

int sk_puts(fno, buf)
/*++++++++++++++++
.PURPOSE  Write a line (followed by a \n)
.RETURNS  Number of bytes written
.REMARKS  Several trials if necessary
- -----------------*/
	int fno;	/* IN: File Descriptor */
	char *buf;	/* IN: Buffer to send  */
{
	char *p;
	int i;

  p = buf + strlen(buf); *p = '\n';
  i = sk_write(fno, buf, 1 + (p-buf));
  *p = 0;
  return (i);
}

int sk_putl(fno, val)
/*++++++++++++++++
.PURPOSE  Write a 32-bit integer on a socket
.RETURNS  Number of bytes written
- -----------------*/
	int fno;        /* IN: opened socket */
	int val;	/* IN: Value to write */
{
	int nval;
	int  stat;

#ifdef DEBUG
	logl("sk_putl", val);
#endif
  nval = htonl(val);
  stat = sk_write(fno, &nval, sizeof(nval));
  return (stat != 4 ? -1 : stat);
}

int sk_close(fno)
/*++++++++++++++++
.PURPOSE  Close the opened socket
.RETURNS  -1 = error
- -----------------*/
	int fno;        /* IN: socket to close */
{
	int  stat;

  stat = close(fno);
  if (stat < 0) err3("sk_close", fno, 0);
  return (stat);
}
