/******************************************************************************
* C Source: GetImage                                                          
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    This file contains the main program and associated functions for         
*    accessing compressed image data stored on the CD-ROM set of the          
*    Digitized Sky Survey.  Data describing the sub-images to extract are     
*    read from the user interactively or from a file, and then loaded into a  
*    set of data structures. These data structures are then used to drive the 
*    actual extraction of the data from the CD-ROM discs.                     
*                                                                             
*    The various data structures used are:                                    
*                                                                             
*       Plate: Structure containing the most basic information about a plate: 
*          Id, Region, right ascension, declination, epoch, survey, and the   
*          number of the CD-ROM disc on which it's located. (See the files    
*          plate.h and plate.c.)                                              
*                                                                             
*       Plate_List: Linked list of plate data structures. (See the files      
*          platelst.h and platelst.c.)                                        
*                                                                             
*       Field: Structure containing information about a field to be           
*          extracted: Name for the field, right ascension, declination        
*          width, and height. (See the files field.h and field.c.)            
*                                                                             
*       Field_List: Linked list of field data structures. (See the files      
*          fieldlst.h and fieldlst.c.)                                        
*                                                                             
*       Image_Section: Structure containing data about an image section from  
*          a plate image: information about the plate, ie, a plate data       
*          structure, width, height and distance from the plate center. (See  
*          the files imgsectn.h and imgsectn.c)                               
*                                                                             
*       Extraction: Structure containing information about a field to be      
*          extracted from a plate: information about the field of the         
*          extraction, ie, a field data structure, and information about the  
*          image section for the plate of the extraction, ie, the image       
*          section data structure. (See the files extrctn.h and extrctn.c.)   
*                                                                             
*       Extraction_List: Linked list of extraction data structures. (See the  
*          files extctlst.h and extctlst.c.)                                  
*                                                                             
*       Extraction_Set: Linked list of extraction list data structures. Here, 
*           each extraction list in the extraction set contains data for all  
*           the extractions from a single CD-ROM disc. (See the files         
*           extctset.h and extctset.c.)                                       
*                                                                             
* Module List:                                                                
*    main                                                                     
*       Main function for GetImage. Performs initialization, extracts images  
*       according to command line options and user input, if any, and then    
*       cleans up after itself.                                               
*    Add_Field_to_Extraction_Set                                              
*       Get extraction info for a field, and insert the data into an          
*       extraction set.                                                       
*    Close_User_Input                                                         
*       Closes input stream of data from the user, either a file, or stdin.   
*    Close_User_Output                                                        
*       Closes output stream of data to the user, either a file, or stdout.   
*    constructDiscRootDir                                                     
*       Creates a string containing the root directory of a CD-ROM disc in    
******************************************************************************
									      
/* *** GetImage Introduction ***                                           2 *
									      
									      
/*****************************************************************************
*       the file system depending on if the access is through a jukebox.      
*    createVMS_CDROMdismountCommand                                           
*       Creates a string containing a command for dismounting a CD-ROM disc   
*       on an OpenVMS system.                                                 
*    createVMS_CDROMmountCommand                                              
*       Creates a string containing a command for mounting a CD-ROM disc on   
*       an OpenVMS system.                                                    
*    createUNIX_CDROMdismountCommand                                          
*       Creates a string containing a command for dismounting a CD-ROM disc   
*       on a UNIX system.                                                     
*    createUNIX_CDROMmountCommand                                             
*       Creates a string containing a command for mounting a CD-ROM disc on a 
*       UNIX system.                                                          
*    Delete_Options                                                           
*       Free memory allocated the structure holding command line options.     
*    dismountCDROM                                                            
*       Dismount a CD-ROM disc according to the operating system.             
*    Extract_Disc_List_HHH_Headers                                            
*       Extract only the headers of GEIS images from a list of extractions,   
*       all for images from the same CD-ROM disc.                             
*    Extract_Disc_List_Images                                                 
*       Extract images from a list of extractions, all from the same CD-ROM   
*       disc.                                                                 
*    Extract_Fields_from_File                                                 
*       Extract images for fields read from a file.                           
*    Extract_Fields_Interactively                                             
*       Extract images for fields entered interactively.                      
*    Extract_Images_from_Set                                                  
*       Extract images from a set of extraction lists.                        
*    Extracting_Images                                                        
*       Indicates if GetImage has been instructed to perform actuall image    
*       extractions as specified on the command line invoking GetImage.       
*    Get_Cand_Plates_Extraction_Info                                          
*       Get extraction information of field from a set of candidate plates.   
*    Get_Disc_Error_Action                                                    
*       Determine action to take if a CD-ROM disc can't be found.             
*    Get_Extractions_from_File                                                
*       Extract images from extraction info read from a file.                 
*    Get_GetImage_Options                                                     
*       Get command line options for GetImage.                                
*    Make_Candidate_Plate_List                                                
*       Make a list of plates that may contain a requested field.             
*    Make_GetImage_Image_File_Root                                            
*       Make root for image file names.                                       
*    Make_Jukebox_Path_File_Name
*       Create full file name for jukebox path list.
*    mountCDROM                                                               
*       Mounts a CD-ROM disc according to the operating system.               
*    Open_User_Input                                                          
*       Open file or stdin for reading information to describe the image data 
*       to be extracted.                                                      
*    Open_User_Output                                                         
*       Open for or stdout for writing information about the image data that  
*       was extracted.                                                        
*    Print_Help_and_Stop                                                      
*       Print options and stop program.                                       
*    Print_Interactive_Instructions                                           
*       Print instructions for how to use getimage interactively.             
*    Print_Usage                                                              
******************************************************************************/

/* *** GetImage Introduction ***                                           3 */


/******************************************************************************
*       Print the usage of getimage.                                          
*    Read_Jukebox_Path_Table
*       Reads the lookup table for paths to discs in a jukebox		      
*
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 31-MAY-94 by J. Doggett,                                    
*       Updated Get_GetImage_Options function.                                
*    Updated    : 01-JUN-94 by J. Doggett,                                    
*       Updated Print_Help_and_Stop and Print_Usage functions.                
*    Updated    : 20-JUL-94 by J. Doggett,                                    
*       Updated Get_Cand_Plates_Extraction_Info function.                     
*    Updated    : 17-OCT-94 by J. Doggett,                                    
*       Updated createUNIX_CDROMmountCommand, Get_Disc_Error_Action and       
*       mountCDROM functions.                                                 
*    Updated    : 30-NOV-94 by J. Doggett,                                    
*       Updated main function.                                                
*    Redelivered: 30-NOV-94 by J. Doggett,                                    
*       GetImage Version 1.1.                                                 
*                                                                             
*    Updated    : 24-JAN-95 by J. Doggett,                                    
*       Updated Get_GetImage_Options, Print_Help_and_Stop, and Print_Usage    
*          functions.                                                         
*       Added constructDiscRootDir function.                                  
*    Updated    : 30-APR-95 by J. Doggett,                                    
*       Updated main function.                                                
*    Updated    : 01-JUN-95 by J. Doggett,                                    
*       Updated mountCDROM function.                                          
*    Redelivered: 18-AUG-95 by J. Doggett,                                    
*       GetImage Version 1.2.                                                 
*                                                                             
*    Updated    : 20-SEP-95 by J. Doggett,                                    
*       Updated Add_Field_to_Extraction functions.                            
*    Updated    : 28-SEP-95 by J. Doggett,                                    
*       Updated Extract_Fields_from_File, Extract_Field_Interactively, and    
*       Print_Interactive_Instructions functions.                             
*    Updated    : 29-OCT-95 by J. Doggett,                                    
*       Updated constructDiscRootDir function.                                
*    Updated    : 04-JAN-96 by J. Doggett,                                    
*       Updated Get_GetImage_Options.                                         
*    Updated    : 05-JAN-96 by J. Doggett,                                    
*       Updated Get_Cand_Plates_Extraction_Info function.                     
*    Updated    : 27-FEB-96 by J. Doggett,                                    
*       Updated main, Print_Help_and_Stop, and Print_Usage functions.         
*    Updated    : 01-MAY-96 by J. Doggett,                                    
*       Updated Extract_Disc_List_Images function.                            
*    Redelivered: 01-AUG-96 by J. Doggett,                                    
*       GetImage Version 1.3.                                                 
*                                                                             
*    Updated    : 11-SEP-96 by J. Doggett,                                    
*       Removed createDDS_CDROMlabel function because the modification of     
*          GetImage to exclusively use volume labels and no long use          
******************************************************************************/

/* *** GetImage Introduction ***                                           4 */


/******************************************************************************
*          disc numbers has made createDDS_CDROMlabel obsolete.               
*       Redelivered Get_Disc_Error_Action function.                     
*    Updated    : 17-SEP-96 by J. Doggett,                                    
*       Redelivered Print_Help_and_Stop and Print_Usage functions.
*    Updated    : 18-SEP-96 by J. Doggett,                                    
*       Redelivered  Extract_Fields_Interactively, Extract_Fields_from_File,
*          Get_Extractions_from_File, Extract_Images_from_Set, and
*          constructDiscRootDir functions.     
*    Updated    : 25-NOV-96 by J. Doggett,
*       Redelivered Extract_Disc_List_Images function.
*    Updated    : 24-FEB-97 by J. Doggett,
*       Redeliverd Get_GetImage_Options and Delete_Options function.
*    Updated    : 27-FEB-97 by J. Doggett,
*       Redelivered main function.
*       Delivered new functions Make_Jukebox_Path_File_Name, 
*          Read_Jukebox_Path_Table and include file looktab.h required by the
*          new functions.                            
*    Redelivered: xx-xxx-xx by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
*                                                                             
* Copyright (c) 1993-1997, Association of Universities for Research in        
* Astronomy, Inc.  All Rights Reserved.                                       
* Produced under National Aeronautics and Space Administration grant          
* NAG W-2166.                                                                 
******************************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* access function not prototyped in standard place */
#if defined __VMS
#  include <unixio.h>
#elif defined __UNIX
#  include <unixstd.h>
#else
#endif

#include "getimage.h"

#include "astrmcal.h"
#include "astronmy.h"
#include "config.h"
#include "extrctn.h"
#include "extrtlst.h"
#include "extrtset.h"
#include "field.h"
#include "fieldlst.h"
#include "filenams.h"
#include "generic.h"
#include "hdrlist.h"
#include "header.h"
#include "image.h"
#include "imgsectn.h"
#include "imgsnlst.h"
#include "looktab.h"
#include "plate.h"
#include "platelst.h"
#include "writeimg.h"

/******************************************************************************
* Module:                                                                     
*    main                                                                     
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    This is the main function  for GetImage.                                 
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       argc                                                                  
*          int, Value                                                         
*             Standard main argument indicating number of words on the        
*              command line.                                                  
*       argv                                                                  
*          char*, Reference                                                   
*             Standard main argument of an array of pointers to the words on  
*             the command line.                                               
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 30-APR-95 by J. Doggett,                                    
*       Modified to NOT dismount if a jukebox is being used.                  
*       Updated to display GetImage as Version 1.2.                           
*    Redelivered: 18-AUG-95 by J. Doggett,                                    
*       GetImage Version 1.2.                                                 
*                                                                             
*    Updated    : 27-FEB-96 by J. Doggett,                                    
*       Updated to display GetImage as Version 1.3.                           
*       Modified to terminate with exit function rather than return.          
*    Redelivered: 01-AUG-96 by J. Doggett,                                    
*       GetImage Version 1.3.                                                 
*                                                                             
*    Updated    : 11-SEP-96 by J. Doggett,                                    
*       Updated to display GetImage as Version 2.0x.                          
*    Updated    : 18-SEP-96 by J. Doggett,                                    
*       Modified calls to Extract_Fields_Interactively,                       
*          Extract_Fields_from_File, and Get_Extractions_from_File to include 
*          jukebox path list.                                                 
*    Updated    : 17-DEC-96 by J. Doggett,
*       Moved call to Get_GetImage_Options to be first executable action.
*    Updated    : 26-FEB-97 by J. Doggett,                                    
*       Added calls to Read_Jukebox_Path_Table and Del_LookupTbl and associated
*          code to support  it.
*    Redelivered: 24-FEB-97 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/

/* *** GetImage main function ***                                          2 */


/*********************
* main declarations  *
*********************/
      main( argc, argv )
int   argc;
char* argv[];
{
   OPTIONS    Options             ; /* Options obtained from command line.   */
   PLATE_LIST Plate_List          ; /* List of plates in plate data base.    */
   LOOK_TAB   Jukebox_Path_Table;

/*****************************
* main begin executable code *
*****************************/
   Options = Get_GetImage_Options( argc, argv );

   printf("\n\n\t\t\t        GetImage\n");
   printf("\n\t\t\t       Version 2.0x\n");
   printf("\n\t\t        Copyright (c) 1993-1996,\n");
   printf("      Association of Universities ");
   printf("for Research in Astronomy, Inc.\n");
   printf(  "\t\t          All Rights Reserved.\n");

   Plate_List = Read_Plate_List( Options->Plate_List_File );
   if (Options->jukeBox == TRUE)
   {
      Jukebox_Path_Table = Read_Jukebox_Path_Table(Options->jukeBoxLookupFile);
      if (Jukebox_Path_Table == (LOOK_TAB)NULL)
      {
         Delete_Plate_List_Data( Plate_List);
         Close_User_Input(Options);
         Close_User_Output(Options);
         Delete_Options(Options);
	 fprintf(stderr, "Aborting GetImage.\n");
         exit(EXIT_FAILURE);
      }
   }
   Open_User_Input ( Options );
   Open_User_Output( Options );

   switch( Options->User_Input )
   {
      case STDIN_FIELDS:
         Extract_Fields_Interactively(Plate_List, Jukebox_Path_Table, Options);
         break;
      case FIELD_FILE:
         Extract_Fields_from_File( Plate_List, Jukebox_Path_Table, Options );
         break;
      case IMAGE_SECTION_FILE:
         Get_Extractions_from_File( Plate_List, Jukebox_Path_Table, Options );
         break;
      default:
         fprintf( stderr, "Input image sections not yet implemented.\n" );
         exit( EXIT_FAILURE );
   }

   printf("Cleaning up..." );

   Delete_Plate_List_Data( Plate_List );
   if (Options->jukeBox == TRUE) Del_LookupTbl(Jukebox_Path_Table);


/* *** GetImage main function ***                                          2 */


   printf("\n");

   Close_User_Input ( Options );

   Close_User_Output( Options );

   if ((Options->dontUnmountOnExit == FALSE)
                       &&
       (Options->jukeBox           == FALSE)
                       &&
       (Extracting_Images(Options) == TRUE )
      )
      dismountCDROM(Options->Dismount_Command, Options->CDROM_Device);

   printf( "\nGetImage all done.\n\n" );

   Delete_Options( Options );

   exit(EXIT_SUCCESS);

   return;
}
/***************************
* main end executable code *
***************************/

/******************************************************************************
* Module:                                                                     
*    Add_Field_to_Extraction_Set                                              
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Find extractions for a field, and insert them into an extraction set.    
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Field                                                                 
*          FIELD, Reference                                                   
*             Field for which we're getting extractions                       
*       Field_List                                                            
*          FIELD_LIST, Reference                                              
*             List where we'll store the field.                               
*       Master_Plate_List                                                     
*          PLATE_LIST, Reference                                              
*             List of all available plates from which we'll get extractions   
*             for the input field.                                            
*       Image_Section_List                                                    
*          IS_LIST, Reference                                                 
*             List where we'll store info about the image sections for the    
*             extractions of the input field.                                 
*       Header_List                                                           
*          HDR_LIST, Reference                                                
*             List where we'll store the headers for the plates from which    
*             we're extracting the input field.                               
*       Extraction_Set                                                        
*          X_Set, Reference                                                   
*             Set of list where we'll store the extraction info for the input 
*             field.                                                          
*       Options                                                               
*          OPTIONS, Reference                                                 
*             The options obtained from the command line. We use them here to 
*             find the directory with the headers for the plate image files.  
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 07-SEP-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*   Updated     : 20-SEP-95 by J. Doggett,                                    
*      If calibration data not found in batch mode, action to take is now     
*      inferred from batch error option specified by                          
*      Options->Batch_Error_Action.                                           
*    Redelivered: 01-AUG-96 by J. Doggett,                                    
*       GetImage Version 1.3.                                                 
******************************************************************************/


/* *** Add_Field_to_Extraction ***                                         2 */


/*******************************************
* Add_Field_to_Extraction_Set declarations *
*******************************************/
void Add_Field_to_Extraction_Set
      ( Field, Field_List, Master_Plate_List, Image_Section_List, Header_List,
            Extraction_Set, Options  )
FIELD      Field             ;
FIELD_LIST Field_List        ;

PLATE_LIST Master_Plate_List ;
IS_LIST    Image_Section_List;
HDR_LIST   Header_List       ;
X_SET      Extraction_Set    ;
OPTIONS    Options           ;
{
   PLATE_LIST Candidate_Plate_List ; /* List of plates that the possibly  *
                                      *    contain the input field.       */
   X_LIST     Field_Extraction_List; /* List of extractions for the input *
                                      *    field.                         */

   char       errorActionChar;
   char       tempChar       ;

/****************************************************
* Add_Field_to_Extraction_Set begin executable code *
****************************************************/
   Add_Field_to_List( Field_List, Field );
   Candidate_Plate_List =
         Make_Candidate_Plate_List( Master_Plate_List, Field );
   Field_Extraction_List =
         Get_Cand_Plates_Extraction_Info( Field               ,
                                          Candidate_Plate_List,
                                          Options             ,
                                          Image_Section_List ,
                                          Header_List
                                        );
   if (Extraction_List_Is_Empty(Field_Extraction_List))
   {
      fprintf(stderr, "Calibration and image data not available for " );
      fprintf(stderr, "field %s...\n", Field->Name );

      if ( ( Options->User_Input == FIELD_FILE         )
                                 ||
           ( Options->User_Input == IMAGE_SECTION_FILE )
         )
      {
         switch ( Options->Batch_Error_Action )
         {
            case SKIP : fprintf(stderr, "Skipping to next field...\n"); break;
            case ABORT:
               Handle_Fatal_Error( "", "Aborting GetImage...");
            default   :
               errorActionChar = ' ';
               while (errorActionChar != 'c')
               {
                  printf("\nEnter a or A to abort GetImage or \n");
                  printf("      c or C to continue...\n");
                  errorActionChar= tolower(getchar());
                  ungetc(errorActionChar, stdin);
                  if (errorActionChar != EOF)
                     while (((tempChar=getchar()) != '\n')

/* *** Add_Field_to_Extraction ***                                         3 */


                                                 &&
                            (tempChar            != EOF  ));
                  if (errorActionChar == 'a')
                     Handle_Fatal_Error("", "Aborted at user's request.");
               }
         }
      }
   }
   else
      Insert_List_into_Extraction_Set( Extraction_Set, Field_Extraction_List );

   Delete_Extraction_List( Field_Extraction_List );
   Delete_Plate_List( Candidate_Plate_List );

   return;
}
/**************************************************
* Add_Field_to_Extraction_Set end executable code *
**************************************************/

/******************************************************************************
* Module:                                                                     
*    Close_User_Input                                                         
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Closes the file where information for data to extract was read.          
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Options                                                               
*          OPTIONS, Reference                                                 
*             Structure with options obtained from the command line.          
*    Input/Output Arguments:                                                  
*          none                                                               
*    Output Arguments:                                                        
*          none                                                               
*                                                                             
* History:                                                                    
*    Created    : 10-DEC-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/********************************
* Close_User_Input declarations *
********************************/
void Close_User_Input( Options )
OPTIONS Options;
{
/*****************************************
* Close_User_Input begin executable code *
*****************************************/
   if ( ( Options->User_Input == FIELD_FILE         )
                            ||
        ( Options->User_Input == IMAGE_SECTION_FILE ) )
      if ( fclose( Options->User_Input_File ) != 0 )
         fprintf(stderr, "Error closing %s.\n",
               Options->User_Input_File_Name);

   return;
}
/***************************************
* Close_User_Input end executable code *
***************************************/

/******************************************************************************
* Module:                                                                     
*    Close_User_Output                                                        
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Closes the path to where information for each of the extractions were be 
*    written. If it's a file, then a message is written to stout.             
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Options                                                               
*          OPTIONS, Reference                                                 
*             Structure with options obtained from the command line.          
*    Input/Output Arguments:                                                  
*          none                                                               
*    Output Arguments:                                                        
*          none                                                               
*                                                                             
* History:                                                                    
*    Created    : 10-DEC-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/*********************************
* Close_User_Output declarations *
*********************************/
void Close_User_Output( Options )
OPTIONS Options;
{
/******************************************
* Close_User_Output begin executable code *
******************************************/
   if ( Options->User_Output == IMAGE_SECTION_FILE )
   {
      if ( fclose( Options->User_Output_File ) == 0 )
         printf( "%s created.\n", Options->User_Output_File_Name);

      else
         fprintf(stderr, "Error closing %s.\n",
               Options->User_Output_File_Name);
   }
   else
      Options->User_Output_File = stdout;

   return;
}
/****************************************
* Close_User_Output end executable code *
****************************************/

/******************************************************************************
* Module:                                                                     
*    constructDiscRootDir                                                     
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Creates a string containing the root directory of a CD-ROM disc in the   
*    file system depending on if the access is through a jukebox.  Note that  
*    memory is dynamically allocated for the string, so it should be "freed"  
*    when the root directory string is no longer needed.                      
*                                                                             
* Return Value:                                                               
*    STRING: Resulting root directory for the disc.                           
*                                                                             
* Arguments:                                                                  
*    Input Arguments:                                                         
*       Options                                                               
*          OPTIONS, Reference                                                 
*             The options obtained from the command line. We use them here to 
*             figure out where a CD-ROM is mounted (or will be mounted) in    
*             the file system.                                                
*    Input/Output Arguments:                                                  
*          none                                                               
*    Output Arguments:                                                        
*          none                                                               
*                                                                             
* History:                                                                    
*    Created    : 24-JAN-95 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 18-AUG-95 by J. Doggett,                                    
*       GetImage Version 1.2.                                                 
*                                                                             
*    Updated    : 29-OCT-95 by J. Doggett,                                    
*       Modified VMS jukebox format to be:                                    
*           <jukeBoxSlotName><jukeBoxSlotNumber>:                             
*    Redelivered: 01-AUG-96 by J. Doggett,                                    
*       GetImage Version 1.3.                                                 
*                                                                             
*    Updated    : 18-SEP-96 by J. Doggett,                                    
	Modified to get path to disc in jukebox from jukebox path table.      
*    Redelivered: 18-SEP-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/************************************
* constructDiscRootDir declarations *
************************************/
STRING   constructDiscRootDir(Options, volume_label, Jukebox_Path_Table)
OPTIONS  Options           ;
STRING   volume_label      ;
LOOK_TAB Jukebox_Path_Table;
{
  STRING rootDir              ; /* Root directory for CD-ROM to be returned.  */
  int    rootDirLength        ;

  STRING jukebox_path;

/* *** constructDiscRootDir ***                                            2 */


/*********************************************
* constructDiscRootDir begin executable code *
*********************************************/
   if (Options->jukeBox)
   {
      jukebox_path = Find_LookupTbl_Entry(Jukebox_Path_Table, volume_label);
         if (jukebox_path == (char *)NULL)
	    jukebox_path = "";

      rootDirLength = strlen(jukebox_path);
      rootDir = (STRING)Allocate_Memory(rootDirLength + 1);
      strcpy(rootDir, jukebox_path);

   }
   else
   {
      rootDirLength = strlen(Options->Image_Data_Root_Dir);
      rootDir = (STRING)Allocate_Memory(rootDirLength + 128);
      strcpy(rootDir, Options->Image_Data_Root_Dir);
   }

   return rootDir;
}
/*******************************************
* constructDiscRootDir end executable code *
*******************************************/

/******************************************************************************
* Module:                                                                     
*    createVMS_CDROMdismountCommand                                           
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Creates a string to send to an OpenVMS operating system for dismounting  
*    a CD-ROM disc.  The basic command with any options is concatenated with  
*    the device name for the CD-ROM drive. The resulting  tring is then       
*    returned.                                                                
*                                                                             
* Return Value:                                                               
*    STRING: Resulting OpenVMS dismount command.                              
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       BaseCommand                                                           
*          STRING, Reference                                                  
*             Basic command with any options for dismounting a CD-ROM disc on 
*             a OpenVMS system.                                               
*       Device                                                                
*          STRING, Reference                                                  
*             Device name of the CD-ROM drive on the OpenVMS system.          
*    Input/Output Arguments:                                                  
*          none                                                               
*    Output Arguments:                                                        
*          none                                                               
*                                                                             
* History:                                                                    
*    Created    : 07-FEB-94 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/**********************************************
* createVMS_CDROMdismountCommand declarations *
**********************************************/
STRING createVMS_CDROMdismountCommand(BaseCommand, Device)
STRING BaseCommand;
STRING Device     ;
{
   STRING dismountCommand; /* Resulting command string to be returned */

/*******************************************************
* createVMS_CDROMdismountCommand begin executable code *
*******************************************************/
   dismountCommand = (STRING)Allocate_Memory(strlen(BaseCommand) +
                                          1                   + /* space */
                                          strlen(Device     ) +
                                          1                  ); /* NULL  */
   strcpy(dismountCommand, BaseCommand);
   strcat(dismountCommand, " "        );
   strcat(dismountCommand, Device     );

   return dismountCommand;
}



/* *** createVMS_CDROMdismountCommand ***                                  2 */


/*****************************************************
* createVMS_CDROMdismountCommand end executable code *
*****************************************************/

/******************************************************************************
* Module:                                                                     
*    createVMS_CDROMmountCommand                                              
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Creates a string to send to OpenVMS operating system for mounting a      
*    CD-ROM disc.  The basic command with any options is concatenated with    
*    the device name for the CD-ROM drive and the volume label of the disc to 
*    be mounted.  The resulting string is then returned.                      
*                                                                             
* Return Value:                                                               
*    STRING: Resulting OpenVMS mount command.                                 
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       BaseCommand                                                           
*          STRING, Reference                                                  
*             Basic command with any options for mounting a CD-ROM disc on an 
*             OpenVMS system.                                                 
v*       Device                                                               
*          STRING, Reference                                                  
*             Device name of the CD-ROM drive on the OpenVMS system.          
*       volumeLabel                                                           
*          STRING, Reference                                                  
*             Volume label for the CD-ROM disc to be mounted.                 
*    Input/Output Arguments:                                                  
*          none                                                               
*    Output Arguments:                                                        
*          none                                                               
*                                                                             
* History:                                                                    
*    Created    : 10-JAN-94 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/*******************************************
* createVMS_CDROMmountCommand declarations *
*******************************************/
STRING createVMS_CDROMmountCommand(BaseCommand, Device, volumeLabel)
STRING BaseCommand;
STRING Device     ;
STRING volumeLabel;
{
   STRING mountCommand; /* Resulting command string to be returned */

/****************************************************
* createVMS_CDROMmountCommand begin executable code *
****************************************************/
   mountCommand = (STRING)Allocate_Memory(strlen(BaseCommand) +
                                          1                   + /* space */
                                          strlen(Device     ) +
                                          1                   + /* space */
                                          strlen(volumeLabel) +
                                          1                  ); /* NULL  */
   strcpy(mountCommand, BaseCommand);
   strcat(mountCommand, " "        );

/* *** createVMS_CDROMountCommand ***                                      2 */


   strcat(mountCommand, Device     );
   strcat(mountCommand, " "        );
   strcat(mountCommand, volumeLabel);

   return mountCommand;
}
/**************************************************
* createVMS_CDROMmountCommand end executable code *
**************************************************/

/******************************************************************************
* Module:                                                                     
*    createUNIX_CDROMdismountCommand                                          
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Creates a string to send to UNIX operating system for dismounting a      
*    CD-ROM disc. The basic command with any options is concatenated with the 
*    device name for the CD-ROM drive.  The resulting string is then          
*    returned.                                                                
*                                                                             
* Return Value:                                                               
*    STRING: Resulting UNIX dismount command.                                 
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       BaseCommand                                                           
*          STRING, Reference                                                  
*             Basic command with any options for mounting CD-ROM disc on a    
*             UNIX system.                                                    
*       Device                                                                
*          STRING, Reference                                                  
*             Device name of the CD-ROM drive on the UNIX system.             
*    Input/Output Arguments:                                                  
*          none                                                               
*    Output Arguments:                                                        
*          none                                                               
*                                                                             
* History:                                                                    
*    Created    : 07-FEB-94 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/***********************************************
* createUNIX_CDROMdismountCommand declarations *
***********************************************/
STRING createUNIX_CDROMdismountCommand(BaseCommand, Device)
STRING BaseCommand;
STRING Device     ;
{
   STRING dismountCommand; /* Resulting command string to be returned */

/********************************************************
* createUNIX_CDROMdismountCommand begin executable code *
********************************************************/

   dismountCommand = (STRING)Allocate_Memory(strlen(BaseCommand) +
                                          1                   + /* space */
                                          strlen(Device     ) +
                                          1                  ); /* NULL  */
   strcpy(dismountCommand, BaseCommand);
   strcat(dismountCommand, " "        );
   strcat(dismountCommand, Device     );

   return dismountCommand;
}


/* *** createUNIX_CDROMdismountCommand ***                                 2 */


/******************************************************
* createUNIX_CDROMdismountCommand end executable code *
******************************************************/


/******************************************************************************
* Module:                                                                     
*    createUNIX_CDROMmountCommand                                             
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Creates a string to send to a UNIX operating system for mounting a       
*    CD-ROM disc.  The basic command with any options is concatenated with    
*    the device name for the CD-ROM drive and the volume label of the disc to 
*    be mounted. The resulting string is then returned.                       
*                                                                             
* Return Value:                                                               
*    STRING: Resulting UNIX mount command.                                    
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       BaseCommand                                                           
*          STRING, Reference                                                  
*             Basic command with any options for mounting a CD-ROM disc on a  
*             UNIX system.                                                    
*       Device                                                                
*          STRING, Reference                                                  
*             Device name of the CD-ROM drive on the UNIX system.             
*       volumeLabel                                                           
*          STRING, Reference                                                  
*             Volume label for the CD-ROM disc to be mounted.                 
*    Input/Output Arguments:                                                  
*          none                                                               
*    Output Arguments:                                                        
*          none                                                               
*                                                                             
* History:                                                                    
*    Created    : 10-JAN-94 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 17-OCT-94 by J. Doggett,                                    
*       Modified to facilitate ss_mount program for use in managing the       
*       mounting of CD-ROMs on UNIX systems.                                  
*    Redelivered: 30-NOV-94 by J. Doggett,                                    
*       GetImage Version 1.1.                                                 
******************************************************************************/
/********************************************
* createUNIX_CDROMmountCommand declarations *
********************************************/
STRING createUNIX_CDROMmountCommand(BaseCommand, Device, mountPoint
#ifdef USE_SSMOUNT
, Label
#endif
)
STRING BaseCommand;
STRING Device     ;
STRING mountPoint ;
#ifdef USE_SSMOUNT
STRING Label      ;
#endif

/* *** createUNIX_CDROMmountCommand ***                                    2 */


{
   STRING mountCommand; /* Resulting command string to be returned */

/*****************************************************
* createUNIX_CDROMmountCommand begin executable code *
*****************************************************/

   mountCommand = (STRING)Allocate_Memory(strlen(BaseCommand) +
                                          1                   + /* space */
                                          strlen(Device     ) +
                                          1                   + /* space */
                                          strlen(mountPoint ) +
#ifdef USE_SSMOUNT
                                          1                   + /* space */
                                          strlen(Label      ) +
#endif
                                          1                 );  /* NULL  */
   strcpy(mountCommand, BaseCommand);
   strcat(mountCommand, " "        );
   strcat(mountCommand, Device     );
   strcat(mountCommand, " "        );
   strcat(mountCommand, mountPoint );
#ifdef USE_SSMOUNT
   strcat(mountCommand, " "        );
   strcat(mountCommand, Label      );
#endif

   return mountCommand;
}
/***************************************************
* createUNIX_CDROMmountCommand end executable code *
***************************************************/

/******************************************************************************
* Module:                                                                     
*    Delete_Options                                                           
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Free memory previously allocated options structure including separately  
*       allocated space for the strings it points to.                         
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Options                                                               
*          OPTIONS, Reference                                                 
*             The options structure we're deleting from memory.               
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 01-SEP-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*
*    Updated    : 17-DEC-96 by J. Doggett,
*       Removed freeing of CDROM_Label_Prefix memory as the fields has
*       been removed.
*    Updated    : 24-FEB-97 by J. Doggett,
*       Added case to free jukebox lookup file.
*    Redelivered: 24-FEB-97 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/******************************
* Delete_Options declarations *
******************************/
void  Delete_Options( Options )
OPTIONS Options;
{
/***************************************
* Delete_Options begin executable code *
***************************************/
   if ( Options == NULL )
      return;

   if ( Options->Command_String != NULL )
      free( Options->Command_String );

   if ( Options->User_Input_File_Name != NULL )
      free( Options->User_Input_File_Name );

   if ( Options->User_Output_File_Name != NULL )
      free( Options->User_Output_File_Name );

   if ( Options->Header_Directory != NULL )

/* *** Delete_Options ***                                                  2 */


      free( Options->Header_Directory );

   if ( Options->Header_Extension != NULL )
      free( Options->Header_Extension );

   if ( Options->Plate_List_File != NULL )
      free( Options->Plate_List_File );

   if ( Options->Image_Data_Root_Dir != NULL )
      free( Options->Image_Data_Root_Dir );

   if ( Options->Mount_Command != NULL )
      free( Options->Mount_Command );

   if ( Options->Dismount_Command != NULL )
      free( Options->Dismount_Command );

   if ( Options->CDROM_Device != NULL )
      free( Options->CDROM_Device );

   if ( Options->CDROM_Mount_Point != NULL )
      free( Options->CDROM_Mount_Point );

   if ( Options->jukeBoxLookupFile != NULL)
      free( Options->jukeBoxLookupFile);

   free( Options );

   return;
}
/*************************************
* Delete_Options end executable code *
*************************************/

/******************************************************************************
* Module:                                                                     
*    dismountCDROM                                                            
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Dismounts a CD-ROM disc. The command string is constructed given the     
*    base system command and device name for the CD-ROM drive depending on    
*    the operating system. The command string is then passed the operating    
*    system "system" function.                                                
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       BaseCommand                                                           
*          STRING, Reference                                                  
*             Basic command with any options for dismounting CD-ROM disc.     
*       Device                                                                
*          STRING, Reference                                                  
*             Device name of the CD-ROM drive.                                
*    Input/Output Arguments:                                                  
*          none                                                               
*    Output Arguments:                                                        
*          none                                                               
*                                                                             
* History:                                                                    
*    Created    : 10-JAN-94 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/*****************************
* dismountCDROM declarations *
*****************************/
void   dismountCDROM(BaseCommand, Device)
STRING BaseCommand;
STRING Device     ;
{
   STRING dismountCommand; /* String with mount command to send to the */
                           /* system.                                  */

/**************************************
* dismountCDROM begin executable code *
**************************************/
#if defined __VMS
   dismountCommand = createVMS_CDROMdismountCommand(BaseCommand, Device);

#elif defined __UNIX
   dismountCommand = createUNIX_CDROMdismountCommand(BaseCommand, Device);

#else
   fprintf(stderr,"Unknown file system error in dismountCDROM");
   exit(EXIT_FAILURE);

#endif


/* *** dismountCDROM ***                                                   2 */


   strcpy(dismountCommand, BaseCommand);
   strcat(dismountCommand, " "        );
   strcat(dismountCommand, Device     );

   printf("Dismounting %s...", Device);
   fflush(stdout);

   system(dismountCommand);
   printf("\n");

   free(dismountCommand);

   return;
}
/************************************
* dismountCDROM end executable code *
************************************/

/******************************************************************************
* Module:                                                                     
*    Extract_Disc_List_HHH_Headers                                            
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Extract GEIS headers from a list of extractions, all for images from the 
*    same CD-ROM disc.  This is analogous to the Extract_Disc_List_Images     
*    function.                                                                
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       List                                                                  
*          X_LIST, Reference                                                  
*             Linked list of extractions.                                     
*       Header_List                                                           
*          HDR_LIST, Reference                                                
*             List of header for all extractions.                             
*       Options                                                               
*          OPTIONS, Value                                                     
*             Structure containing options obtained from the command line.    
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 08-DEC-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/*********************************************
* Extract_Disc_List_HHH_Headers declarations *
*********************************************/
void     Extract_Disc_List_HHH_Headers( Disc_List, Header_List )
X_LIST   Disc_List  ;
HDR_LIST Header_List;
{
   XL_NODE    Disc_List_Node;
   EXTRACTION Extraction    ;
   STRING     imageFileRoot ; /* Root for image file name to create      */
   HEADER     Plate_Header  ;
   HEADER     Image_Header  ;

/******************************************************
* Extract_Disc_List_HHH_Headers begin executable code *
******************************************************/
   Disc_List_Node = Disc_List->Top;

   while ( Disc_List_Node != NULL )
   {
      Extraction = ( EXTRACTION )( Disc_List_Node->Data );

      Plate_Header = Get_Header_from_List

/* *** Extract_Disc_List_HHH_Headers ***                                   2 */


                              ( Header_List, Extraction->Plate->Id );
      Image_Header = Construct_Image_Header( Plate_Header, Extraction, NULL );

      imageFileRoot = Make_GetImage_Image_File_Root( Extraction );

      if (HHH_File_Exists(imageFileRoot))
      {
#if defined __VMS
         fprintf (stderr, "A new version of %s.hhh will be created.\n",
               imageFileRoot);
         Write_HHH_Header( Image_Header, imageFileRoot );
#elif defined __UNIX
         fprintf(stderr, "%s.hhh already exists for field %s on plate %s.\n",
               imageFileRoot, Extraction->Field->Name, Extraction->Plate->Id);
         fprintf(stderr, "Header file not created...\n" );
#else
         fprintf(stderr,
               "Unknown file system error in Extract_Disc_List_HHH_Headers");
         exit(EXIT_FAILURE);
#endif
      }
      else
         Write_HHH_Header( Image_Header, imageFileRoot );

      free( Image_Header  );
      free( imageFileRoot );

      Disc_List_Node = Disc_List_Node->Next;
   }
   return;
}
/****************************************************
* Extract_Disc_List_HHH_Headers end executable code *
****************************************************/

/******************************************************************************
* Module:                                                                     
*    Extract_Disc_List_Images                                                 
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Extract images from a list of extractions, all from the same CD-ROM      
*    disc.  If user gave option of only writing the headers for GEIS files,   
*    then Extract_Disc_List_HHH_Headers is called, rather than extracting     
*    images.  NOTE:  The extraction list is NOT checked to verify that all    
*    the extractions are indeed from the same CD-ROM disc.                    
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       List                                                                  
*          X_LIST, Reference                                                  
*             Linked list of extractions.                                     
*       Header_List                                                           
*          HDR_LIST, Reference                                                
*             List of header for all extractions.                             
*       Options                                                               
*          OPTIONS, Value                                                     
*             Structure containing options obtained from the command line.    
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 07-SEP-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 25-JAN-94 by J. Doggett,                                    
*       Modified to use the new constructDiscRootDir function to determine    
*       the parent to the compressed image directory.  This facilitates the   
*       use of a CD-ROM jukebox.                                              
*    Redelivered: 18-AUG-95 by J. Doggett,                                    
*       GetImage Version 1.2.                                                 
*                                                                             
*    Updated    : 19-JAN-96 by J. Doggett,                                    
*       Modified to optionally create image with user specified file name.    
*    Updated    : 01-MAY-96 by J. Doggett,                                    
*       Removed check for plate's disc being available because the marker     
*       for unavailable discs was using 0 for the disc number.  However,      
*       Disc 0 is used for the 100x demo set, and no plate index files have   
*       been made with unavailable discs with the current require format for  
*       plate index files.                                                    
*    Redelivered: 01-AUG-96 by J. Doggett,                                    
*       GetImage Version 1.3.                                                 
*                                                                             
*    Updated    : 11-SEP-96 by J. Doggett,                                    
*       Modified for change in calling sequence to constructDiscRootDir and   
******************************************************************************/

/* *** Extract_Disc_List_Images ***                                        2 */

/******************************************************************************
*          and Get_Disc_Error_Action functions.                               
*       Replaced use of disc number by volume label.                          
*    Updated    : 18-SEP-96 by J. Doggett,                                    
*       Added jukebox path list to argument list for support of final jukebox 
*          scheme.                                                            
*       Added jukebox path list and disc volume label to arguments of         
*          constructDiscRootDir.                                          
*    Updated    : 25-NOV-96 by J. Doggett,                                    
*       Modified to pass calibration file name to Extract_Image and to abort
*       extraction if VMS FITS file already exists.
*    Updated    : 12-DEC-96 by J. Doggett,
*       Modified to pass NULL to extract image routine for calibration file
*       when the calibration file name is actuall "no_cal_file".
*    Redelivered: xx-xxx-xx by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/****************************************
* Extract_Disc_List_Images declarations *
****************************************/
void     Extract_Disc_List_Images
               ( Disc_List, Header_List, Jukebox_Path_Table, Options )
X_LIST   Disc_List         ;
HDR_LIST Header_List       ;
LOOK_TAB Jukebox_Path_Table;
OPTIONS  Options           ;
{

   XL_NODE    Disc_List_Node;
   STRING     discRootDir   ;
   EXTRACTION Extraction    ;
   HEADER     Plate_Header  ;
   STRING     imageFileRoot ;
   IMAGE_DATA Image         ;
   HEADER     Image_Header  ;
   STRING     calibrationFile=(STRING)NULL;

   char *p;

/*************************************************
* Extract_Disc_List_Images begin executable code *
*************************************************/
   if ((Options->Format == HHD)
                  &&
       (Options->GEIS_Header_Only == TRUE)
      )
   {
      Extract_Disc_List_HHH_Headers(Disc_List, Header_List);
      return;
   }

   Disc_List_Node = Disc_List->Top;
   Extraction = ( EXTRACTION )( Disc_List_Node->Data );

   while ( Disc_List_Node != NULL )
   {
     discRootDir =
       constructDiscRootDir
       (Options, Extraction->Plate->volumeLabel, Jukebox_Path_Table);

      Extraction = ( EXTRACTION )( Disc_List_Node->Data );
      Plate_Header = Get_Header_from_List
                              ( Header_List, Extraction->Plate->Id );

      /*
       *  Quick and dirty fix for taking nfs jukeboxes into account
       */

      if ( !strcmp(Options->Mount_Command, "cd") ){
	  /* char *p; */
	  int disc;
	  
	  p = strrchr(Extraction->Plate->volumeLabel, '_');
	  if(p) {
	      strcat(discRootDir, p+1);
	      strcat(discRootDir, "/");
	  }
      }

	/*
	 * End QDF
	 */

      if ( Plate_Directory_Exists

/* *** Extract_Disc_List_Images ***                                        3 */


          ( Plate_Header->plate_name, discRootDir  ) == TRUE )
      {
         if (strlen(Options->outputImageName) == 0)
            imageFileRoot = Make_GetImage_Image_File_Root(Extraction);
         else
            imageFileRoot = Options->outputImageName;

         if (Options->extractCalibrations)
	 {
	    if (strcmp(Extraction->Plate->calibrationFile, "no_cal_file")
	          != 0)
	       calibrationFile = Build_Full_File_Name
	                              (Options->Header_Directory, "",
				       Extraction->Plate->calibrationFile);
	    else
	       calibrationFile = (STRING)NULL;
	 }
	 else
	    calibrationFile = (STRING)NULL;

         if (Image_File_Exists(imageFileRoot, Options->Format))
         {
#if defined __VMS
            if (Options->Format == HHD)
	    {
               fprintf(stderr,
                  "A new version of the image, %s.hhh/hhd, will be created.\n",
                  imageFileRoot );
               Extract_Image(Extraction, Plate_Header,
                 discRootDir, imageFileRoot, Options->Format, calibrationFile);
	    }
            else
	    {
               fprintf(stderr, "%s.fits ", imageFileRoot);

	       fprintf(stderr, "already exists " );
	       fprintf(stderr, "for field %s on plate %s.\n",
		       Extraction->Field->Name, Extraction->Plate->Id);
	       fprintf(stderr, "Skipping extraction...\n" );
	    }
#elif defined __UNIX
            if (Options->Format == HHD)
               fprintf(stderr, "%s.hhh or %s.hhd ",
                       imageFileRoot, imageFileRoot);
            else
               fprintf(stderr, "%s.fits ", imageFileRoot);

            fprintf(stderr, "already exists " );
            fprintf(stderr, "for field %s on plate %s.\n",
                     Extraction->Field->Name, Extraction->Plate->Id);
            fprintf(stderr, "Skipping extraction...\n" );
#else
            fprintf(stderr,
                  "Unknown file system error in Extract_Disc_List_Images");
            exit(EXIT_FAILURE);
#endif
         }
         else
            Extract_Image(Extraction, Plate_Header,
                 discRootDir, imageFileRoot, Options->Format, calibrationFile);


/* *** Extract_Disc_List_Images ***                                        4 */


         if (strlen(Options->outputImageName) == 0)
            free(imageFileRoot);

         Disc_List_Node = Disc_List_Node->Next;
      }
      else
      {
         switch(Get_Disc_Error_Action(Extraction->Plate->volumeLabel, Options))
         {
            case SKIP :
              Disc_List_Node = NULL;
              printf
		    ("Skipping Disc %s...\n"       ,
		     Extraction->Plate->volumeLabel);
              break;
            case RETRY:
              Disc_List_Node = Disc_List_Node;
              break;
            default:
              Disc_List_Node = Disc_List_Node;
         }
      }
   }

   if (calibrationFile != (STRING)NULL)
      free(calibrationFile);

   free(discRootDir);

   return;
}
/***********************************************
* Extract_Disc_List_Images end executable code *
***********************************************/

/******************************************************************************
* Module:                                                                     
*    Extract_Fields_from_File                                                 
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Read field information from a file, then extract image data for each the 
*    fields by CD-ROM disc.                                                   
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Argument                                                           
*       Plate_List                                                            
*          PLATE_LIST, Reference                                              
*             List of available plates from which we can extract fields.      
*       Options                                                               
*          OPTIONS, Value                                                     
*             Structure containing options obtained from the command line.    
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 17-AUG-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 28-SEP-95 by J. Doggett,                                    
*       Modified to reflect increase in limit of field name size.             
*    Redelivered: 01-AUG-96 by J. Doggett,                                    
*       GetImage Version 1.3.                                                 
*                                                                             
*    Updated    : 18-SEP-96 by J. Doggett,                                    
*       Added jukebox path list to argument list for support of final jukebox 
*          scheme and to call to Extract_Images_from_Set.		      
*    Redelivered: 18-SEP-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/****************************************
* Extract_Fields_from_File declarations *
****************************************/
void       Extract_Fields_from_File( Plate_List, Jukebox_Path_Table, Options )
PLATE_LIST Plate_List        ;
LOOK_TAB   Jukebox_Path_Table;
OPTIONS    Options           ;
{
   FIELD      Field               ; /* Field data read from input file       */
   FIELD_LIST Field_List          ; /* List of all the fields                */
   PLATE_LIST Candidate_Plate_List; /* List of candidate plates for a field. */
   X_LIST     Extraction_List     ; /* All extractions from single disc.     */
   X_SET      Extraction_Set      ; /* Set of all extractions                */
   HDR_LIST   Header_List         ; /* Header for all plates extracting from */
   IS_LIST    Image_Section_List  ; /* List image sections for extractions   */


/* *** Extract_Fields_from_File ***                                        2 */


   char Output_Headings   [96];
   char Heading_Underlines[96];

   strcpy(Output_Headings   ,
         "Field                  RA(J2000)   Dec(J2000)  Region PlId ");
   strcat(Output_Headings   ,
         "  Epoch    X     Y   Width  Hght Dsc");
   strcpy(Heading_Underlines,
         "----                 ------------ ------------ ------ ---- ");
   strcat(Heading_Underlines,
         "-------- ----- ----- ----- ----- ---");

/*************************************************
* Extract_Fields_from_File begin executable code *
*************************************************/
   if ( Options->User_Output == STDOUT_IMAGE_SECTIONS )
      printf( "\n\n%s\n%s\n", Output_Headings, Heading_Underlines );

   Field_List         = Create_Empty_Field_List        ();
   Image_Section_List = Create_Empty_Image_Section_List();
   Header_List        = Create_Empty_Header_List       ();
   Extraction_Set     = Create_Empty_Extraction_Set    ();

   Field = Read_Field_Info( Options->Equinox, Options->User_Input_File );
   while( ( ( int )Field != EOF ) && ( ( int )Field != ( int )NULL ) )
   {

      Add_Field_to_Extraction_Set( Field             ,
                                   Field_List        ,
                                   Plate_List        ,
                                   Image_Section_List,
                                   Header_List       ,
                                   Extraction_Set    ,
                                   Options
                                 );

      Field = Read_Field_Info( Options->Equinox, Options->User_Input_File );
   }

   if ( Extraction_Set->Top != NULL )
         Extract_Images_from_Set
               ( Extraction_Set, Header_List, Jukebox_Path_Table, Options );
   else
      fprintf( stderr, "\nNo images found to extract.\n" );

   Delete_Field_List_Data        ( Field_List         );
   Delete_Image_Section_List_Data( Image_Section_List );
   Delete_Header_List_Data       ( Header_List        );
   Delete_Extraction_Set_Lists   ( Extraction_Set     );

   return;
}
/***********************************************
* Extract_Fields_from_File end executable code *
***********************************************/

/******************************************************************************
* Module:                                                                     
*    Extract_Fields_Interactively                                             
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Read field information from user interactively, and extract images as we 
*    go.                                                                      
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Argument                                                           
*       Plate_List                                                            
*          PLATE_LIST, Reference                                              
*             List of available plates from which we can extract fields.      
*       Options                                                               
*          OPTIONS, Value                                                     
*             Structure containing options obtained from the command line.    
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 17-AUG-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 28-SEP-95 by J. Doggett,                                    
*       Modified to reflect increase in limit of field name size.             
*    Redelivered: 01-AUG-96 by J. Doggett,                                    
*       GetImage Version 1.3.                                                 
*                                                                             
*    Updated    : 18-SEP-96 by J. Doggett,                                    
*       Added jukebox path list to argument list for support of final jukebox 
*          scheme and to call to Extract_Images_from_Set.		      
*    Redelivered: 18-SEP-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/********************************************
* Extract_Fields_Interactively declarations *
********************************************/
void       Extract_Fields_Interactively
                 ( Plate_List, Jukebox_Path_Table, Options )
PLATE_LIST Plate_List;
LOOK_TAB   Jukebox_Path_Table;
OPTIONS    Options   ;
{
   FIELD      Field             ; /* Field data read from user             */
   FIELD_LIST Field_List        ; /* Field is put into list for processing */
   X_SET      Extraction_Set    ; /* All extractions for the field         */
   IS_LIST    Image_Section_List; /* List image sections for extractions   */
   HDR_LIST   Header_List       ; /* List of headers for plates we're      *
                                   *    extraction from.                   */
   char Output_Headings   [96];

/* *** Extract_Field_Interactively ***                                     2 */


   char Heading_Underlines[96];

   strcpy(Output_Headings   ,
         "Field                  RA(J2000)   Dec(J2000)  Region PlId ");
   strcat(Output_Headings   ,
         "  Epoch    X     Y   Width  Hght Dsc");
   strcpy(Heading_Underlines,
         "----                 ------------ ------------ ------ ---- ");
   strcat(Heading_Underlines,
         "-------- ----- ----- ----- ----- ---");

/*****************************************************
* Extract_Fields_Interactively begin executable code *
*****************************************************/
   Header_List        = Create_Empty_Header_List();
   Print_Interactive_Instructions( Options );

   printf( "\nEnter data for first field: " );

   Field = Read_Field_Info( Options->Equinox, Options->User_Input_File );
   while( ( int )Field != EOF )
   {

      if ( Field == NULL )
         fprintf( stderr, "Error on input, try again...\n\n" );
      else
      {
         if ( Options->User_Output == STDOUT_IMAGE_SECTIONS )
            printf( "\n\n%s\n%s\n", Output_Headings, Heading_Underlines );

         Field_List         = Create_Empty_Field_List        ();
         Image_Section_List = Create_Empty_Image_Section_List();
         Extraction_Set     = Create_Empty_Extraction_Set    ();
         Add_Field_to_Extraction_Set( Field             ,
                                      Field_List        ,
                                      Plate_List        ,
                                      Image_Section_List,
                                      Header_List       ,
                                      Extraction_Set    ,
                                      Options
                                    );
         if ( Extraction_Set->Top != NULL )
            Extract_Images_from_Set
                  ( Extraction_Set, Header_List, Jukebox_Path_Table, Options );
         else
            fprintf( stderr, "No images found for %s!\n", Field->Name );

         Delete_Field_List_Data        ( Field_List         );
         Delete_Image_Section_List_Data( Image_Section_List );
         Delete_Extraction_Set_Lists   ( Extraction_Set     );

      }

      printf( "\nEnter data for next field: " );
      Field = Read_Field_Info( Options->Equinox, Options->User_Input_File );

   }
   printf( "\n\n" );

   Delete_Header_List_Data( Header_List );

   return;

/* *** Extract_Field_Interactively ***                                     3 */
}
/***************************************************
* Extract_Fields_Interactively end executable code *
***************************************************/

/******************************************************************************
* Module:                                                                     
*    Extract_Images_from_Set                                                  
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Extracts the images from a set of extraction lists.                      
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Argument                                                           
*       Extraction_SET                                                        
*          X_SET, Reference                                                   
*             List linking the extraction lists for each disc from which      
*             fields will be extracted.                                       
*       Header_List                                                           
*          HDR_LIST, Reference                                                
*             List of header for all extractions.                             
*       Options                                                               
*          OPTIONS, Value                                                     
*             Structure containing options obtained from the command line.    
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 11-SEP-96 by J. Doggett,                                    
*       Modified to reflect change in calling sequence to                     
*          Get_Lowest_Disc_List_from_set.                                     
*    Updated    : 18-SEP-96 by J. Doggett,                                    
*       Added jukebox path list to argument list for support of final jukebox 
*          scheme and to call to Extract_Disc_List_Images;		      
*    Redelivered: 18-SEP-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/***************************************
* Extract_Images_from_Set declarations *
***************************************/
void     Extract_Images_from_Set
               ( Extraction_Set, Header_List, Jukebox_Path_Table, Options )
X_SET    Extraction_Set      ;
HDR_LIST Header_List         ;
LOOK_TAB   Jukebox_Path_Table;
OPTIONS  Options             ;
{
   XS_NODE Extraction_Set_Node ; /* Node in the extraction set linked list. */
   X_LIST  Extraction_List     ; /* Extraction list in the extraction set.  */
   XL_NODE Extraction_List_Node; /* Node in an extraction list linked list. */
   EXTRACTION Extraction       ; /* Extraction from an extraction list.     */
   X_LIST     Last_Disc_List   ; /* Last list extracted.                    */

/* *** Extract_Images_from_Set ***                                         2 */


/**********************************************
* Extract_Images_from_Set end executable code *
**********************************************/
   printf( "\n" );

   if ( Options->Extract_Images == TRUE )
   {
      Last_Disc_List = (X_LIST)NULL;
      Extraction_List =
                Get_Lowest_Disc_List_from_Set( Extraction_Set, Last_Disc_List);

      while( Extraction_List != NULL )
      {
         Extract_Disc_List_Images
               ( Extraction_List, Header_List, Jukebox_Path_Table, Options );
         Last_Disc_List = Extraction_List;
         Extraction_List =
                Get_Lowest_Disc_List_from_Set( Extraction_Set, Last_Disc_List);
      }

   }
   else
      fprintf( stderr, "Images optionally not extracted.\n" );

   return;
}
/**********************************************
* Extract_Images_from_Set end executable code *
**********************************************/

/******************************************************************************
* Module:                                                                     
*    Extracting_Images                                                        
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Returns TRUE if the current session of GetImage is actually extracting   
*    image data. Normally, this is always the case. However, if the -n        
*    options, or -d with -g options are chosen on the command line, then      
*    no images are being extracted, and FALSE is returned.                    
*                                                                             
* Return Value:                                                               
*    BOOL: TRUE if images are begin extracted, otherwise, FALSE.              
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Options                                                               
*          OPTIONS, Reference                                                 
*             Structure with options obtained from the command line.          
*    Input/Output Arguments:                                                  
*          none                                                               
*    Output Arguments:                                                        
*          none                                                               
*                                                                             
* History:                                                                    
*    Created    : 10-JAN-94 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/*********************************
* Extracting_Images declarations *
*********************************/
BOOL    Extracting_Images(Options)
OPTIONS Options;
{
/******************************************
* Extracting_Images begin executable code *
******************************************/
   if (
       ((Options->Format            == HHD) /* Only extracting header file */
                          &&                /* for GEIS images.            */
        (Options->GEIS_Header_Only == TRUE) /*                             */
       )                                    /*                             */
                          ||                /*        or                   */
       ( Options->Extract_Images == FALSE ) /* Images not being extracted  */
      )                                     /* Then we're not extracting   */
                                            /* images.                     */
      return FALSE;
   else
      return TRUE;
}
/****************************************
* Extracting_Images end executable code *
****************************************/

/******************************************************************************
* Module:                                                                     
*    Get_Cand_Plates_Extraction_Info                                          
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Gets extraction information for a field from its candidate plates.       
*    First, we loop through the candidate plates gathering extraction info    
*    for the field on each plate. If only the best extraction is desired,     
*    then only the extraction closest to a plate center is put into the list. 
*    The resulting extraction data is then written to the output file.        
*                                                                             
* Return Value:                                                               
*    X_LIST: The list of extractions for the input field.                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Field                                                                 
*          FIELD, Reference                                                   
*             Description of a user requested field.                          
*       Candidate Plates                                                      
*          PLATE_LIST, Reference                                              
*             Linked list containing plate centers for plates that may        
*             include the user requested field.                               
*       Options                                                               
*          OPTIONS, Value                                                     
*             Structure containing options obtained from the command line.    
*       Image_Section_List                                                    
*          IS_LIST, Reference                                                 
*             Linked list containing all image sections for extractions.      
*    Input/Output Arguments:                                                  
*       Header_List                                                           
*          HDR_LIST, Reference                                                
*             List of headers for all plates from which we're extracting      
*             images.                                                         
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 20-JUL-94 by J. Doggett,                                    
*       Modified to reflect change in argument list of gethdr function.       
*    Redelivered: 30-NOV-94 by J. Doggett,                                    
*       GetImage Version 1.1.                                                 
*                                                                             
*    Updated    : 05-JAN-96 by J. Doggett,                                    
*       Modified to reflect change in image section field of distance to      
*          centerDistance.                                                    
*       Modified to optionally select plate in which the field is furtherest  
*          from the edge.                                                     
*       Modified to only get extractions from single specified plate if       
*          that option is specified.                                          
*    Redelivered: 01-AUG-96 by J. Doggett,                                    
******************************************************************************/

/* *** Get_Cand_Plates_Extraction_Info ***                                 2 */


/******************************************************************************
*       GetImage Version 1.3.                                                 *
******************************************************************************/
/**********************************************
* Get_Cand_Plates_Extraction_Info declaraions *
**********************************************/
X_LIST     Get_Cand_Plates_Extraction_Info
         ( Field, Candidate_Plates, Options, Image_Section_List, Header_List )
FIELD      Field             ;
PLATE_LIST Candidate_Plates  ;
OPTIONS    Options           ;
IS_LIST    Image_Section_List;
HDR_LIST   Header_List       ;
{
   PL_NODE    Candidate_Node ; /* Node in candidate plate linked list       */
   PLATE      Candidate_Plate; /* Plate from the candidate plate list.      */
   X_LIST     Extraction_List; /* Extraction list for field on cand plates. */
   XL_NODE    Extraction_Node; /* Node in extraction list                   */
   EXTRACTION Extraction     ; /* An extraction from the extraction list  . */
   EXTRACTION Best_Extraction; /* Extraction of candidate plate closest to  *
                                *    to plate center.                       */
   IMAGE_SECTION
              Image_Section  ; /* Image section to be extracted for a field */
   HEADER     Header         ; /* Plate header data                         */
   STRING     Header_File_Name; /* Name of file with header data            */

/********************************************************
* Get_Cand_Plates_Extraction_Info begin executable code *
********************************************************/
   Extraction_List = Create_Empty_Extraction_List();

   Candidate_Node = Candidate_Plates->Top;
   Best_Extraction = NULL;
   while ( Candidate_Node != NULL )
   { /* Loop through candidate plates */

      Candidate_Plate = ( PLATE )( Candidate_Node->Data );

      if ((Options->singlePlate == FALSE                           ) ||
          (strcmp(Options->singlePlateID, Candidate_Plate->Id) == 0)   )

      {
         Header = Get_Header_from_List( Header_List, Candidate_Plate->Id );
         if ( Header == NULL )
         {
            Header_File_Name = Make_Header_File_Name(Options->Header_Directory,
                                                     Candidate_Plate->Region  ,
                                                     Options->Header_Extension
                                                     );
            Header = gethdr(Header_File_Name);
            free(Header_File_Name);
            if ( Header != NULL )
               Add_Header_to_List( Header_List, Header );
         }

         if ( ( int )Header != ( int )NULL )
         {
            Image_Section =
                  Get_Image_Section( Field, Candidate_Plate, Header        );
            Extraction    =
                  Get_Extraction   ( Field, Candidate_Plate, Image_Section );

/* *** Get_Cand_Plates_Extraction_Info ***                                 3 */


         }
         else
         {
            Image_Section = ( IMAGE_SECTION )NULL;
            Extraction    = ( EXTRACTION    )NULL;
         }
      }
      else
      {
         Image_Section = ( IMAGE_SECTION )NULL;
         Extraction    = ( EXTRACTION    )NULL;

      }

      if ( Extraction != NULL )
      { /* Found field on candidate plate */
         if ( Options->Extract_All_Images == TRUE )
         {
            Add_Image_Section_to_List
                             ( Image_Section_List, Extraction->Image_Section );
            Add_Extraction_to_List( Extraction_List, Extraction );
         }
         else
         {
            if ( Best_Extraction == NULL )
               Best_Extraction = Extraction;

            {
               if (Options->plateChoiceFlag == FURTHEST_FROM_EDGE)
               {
                  if (Extraction->Image_Section->edgeDistance >
                      Best_Extraction->Image_Section->edgeDistance)
                  {
                     Delete_Extraction(Best_Extraction);
                     Best_Extraction = Extraction;
                  }
               }
               else if (Options->plateChoiceFlag == CLOSEST_TO_CENTER)
               {
                  if (Extraction->Image_Section->centerDistance <
                      Best_Extraction->Image_Section->centerDistance)
                  {
                     Delete_Extraction(Best_Extraction);
                     Best_Extraction = Extraction;
                  }
               }
            }
         }
      } /* Found field on candidate plate */

      Candidate_Node = Candidate_Node->Next;

   } /* Loop through candidate plates */

   if ( ( Options->Extract_All_Images == FALSE ) &&
                                                  ( Best_Extraction != NULL ) )
   {
      Add_Image_Section_to_List
                        ( Image_Section_List, Best_Extraction->Image_Section );
      Add_Extraction_to_List( Extraction_List, Best_Extraction );
   }

/* *** Get_Cand_Plates_Extraction_Info ***                                 4 */


   Extraction_Node = Extraction_List->Top;
   while ( Extraction_Node != NULL )
   {
      Extraction = ( EXTRACTION )( Extraction_Node->Data );

      Write_Extraction_Information( Extraction, Options->User_Output_File );

      Extraction_Node = Extraction_Node->Next;
   }
   return Extraction_List;
}
/******************************************************
* Get_Cand_Plates_Extraction_Info end executable code *
******************************************************/

/******************************************************************************
* Module:                                                                     
*    Get_Disc_Error_Action                                                    
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Deterimine what action to take if GetImage couldn't find the directory   
*    with plate image data, presumably because the incorrect disc is in the   
*    CD-ROM drive. If input is interactive, then the user is queried for the  
*    action: RETRY (after inserting the correct disc), SKIP the current disc, 
*    or ABORT GetImage. If input is read from a file then the action is taken 
*    from the Batch_Error_Action field of the options structure, one of which 
*    maybe QUERY. If so, then the user is queried as in the interactive case. 
*                                                                             
* Return Value:                                                               
*    ERROR_ACTION: Mnemonic for action to take if the disc couldn't be found. 
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Disc_Volume_Label                                                     
*          STRING, Value                                                      
*             Volume label of disc.                                           
*       Options                                                               
*          OPTIONS, Value                                                     
*             Structure containing options obtained from the command line. We 
*             use this to determine if input is from stdin or a file, and     
*             what action to take if input is from a file.                    
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 08-SEP-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 17-OCT-94 by J. Doggett,                                    
*       Modified to facilitate ss_mount program for use in managing the       
*       mounting of CD-ROMs on UNIX systems.                                  
*    Redelivered: 30-NOV-94 by J. Doggett,                                    
*       GetImage Version 1.1.                                                 
*                                                                             
*    Updated    : 11-SEP-96 by J. Doggett,                                    
*       Modified to remove use of disc numbers in favor of volume labels.     
*    Redelivered: 11-SEP-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/*************************************
* Get_Disc_Error_Action declarations *
*************************************/
ERROR_ACTION Get_Disc_Error_Action( Disc_Volume_Label, Options )
STRING       Disc_Volume_Label;
OPTIONS      Options          ;
{



/* *** get_disc_error_action ***                                           2 */


   int    Action_Character    ; /* Character read from user telling us what
                                 * action to take.
                                 */
   char   Error_Message[ 100 ];
   int    Temp_Char           ;

/**********************************************
* Get_Disc_Error_Action begin executable code *
**********************************************/
   if ( ( Options->User_Input == FIELD_FILE         )
                              ||
        ( Options->User_Input == IMAGE_SECTION_FILE )
      )
   {
      switch ( Options->Batch_Error_Action )
      {
         case SKIP : return SKIP ;
         case ABORT:
               sprintf
                     (Error_Message              ,
		      "Could not locate disc %s!",
		      Disc_Volume_Label          );
               Handle_Fatal_Error( "", Error_Message );
         default   :             ; /* We default to querying the user below */
      }
   }
/* If we got to here, then it's interactive case or, the batch case
 * defaulted to query the user.
 */
   printf( "Could not find disc %s.\n", Disc_Volume_Label);

   printf(
         "Enter <cr>    to dismount current disc and mount disc %s,\n",
	 Disc_Volume_Label                                            );
   printf("       m or M to mount disc %s,\n", Disc_Volume_Label);
   printf("       s or S to skip disc %s,\n", Disc_Volume_Label);
   printf("       a or A to abort GetImage...\n");

   Action_Character = tolower(getchar());
   ungetc(Action_Character, stdin);
   if (Action_Character != EOF)
      while (( (Temp_Char=getchar()) != '\n')
                          &&
             (Temp_Char              != EOF )
            ) ;

   switch ( Action_Character )
   {
      case 's' : return SKIP;
      case 'a' : if ((Options->dontUnmountOnExit == FALSE)
                                     &&
                     (Options->jukeBox           == FALSE)
                                     &&
                     (Extracting_Images(Options) == TRUE )
                    )
                    dismountCDROM
                          (Options->Dismount_Command, Options->CDROM_Device);
                 Handle_Fatal_Error( "", "Aborted at user's request." );
                 return ABORT;
      case EOF : if ((Options->dontUnmountOnExit == FALSE)
                                     &&

/* *** Get_Disc_Error_Action ***                                           3 */


                     (Options->jukeBox           == FALSE)
                                     &&
                     (Extracting_Images(Options) == TRUE )
                    )
                    dismountCDROM
                          (Options->Dismount_Command, Options->CDROM_Device);
                 Handle_Fatal_Error( "", "Aborted at user's request." );
                 return ABORT;
      case 'm' : 
#ifdef USE_SSMOUNT
                 mountCDROM(Options, Disc_Volume_Label);
#else
                 mountCDROM(Options->Mount_Command    ,
                            Options->CDROM_Device     ,
                            Disc_Volume_Label         ,
                            Options->CDROM_Mount_Point);
#endif
                 printf ("\n");
                 return RETRY;
      case '\n': dismountCDROM
                       (Options->Dismount_Command, Options->CDROM_Device);
                 printf(
                      "Now, insert disc %s into the drive and enter <cr>...\n",
                      Disc_Volume_Label);
                 while (( (Temp_Char=getchar()) != '\n')
                                    &&
                        (Temp_Char              != EOF )
                       ) ;
#ifdef USE_SSMOUNT
                 mountCDROM(Options, Disc_Volume_Label);
#else
                 mountCDROM(Options->Mount_Command    ,
                            Options->CDROM_Device     ,
                            Disc_Volume_Label         ,
                            Options->CDROM_Mount_Point);
#endif
                 printf( "\n");
                 return RETRY;

      default  : return RETRY;
   }
}
/********************************************
* Get_Disc_Error_Action end executable code *
********************************************/

/******************************************************************************
* Module:                                                                     
*    Get_Extractions_from_File                                                
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Read information for extractions from a file, then extract the           
*    appropriate images by CD-ROM disc.                                       
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Argument                                                           
*       Options                                                               
*          OPTIONS, Value                                                     
*             Structure containing options obtained from the command line.    
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 25-OCT-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 18-SEP-96 by J. Doggett,                                    
*       Added jukebox path list to argument list for support of final jukebox 
*          scheme and to call to Extract_Images_from_Set.		      
*    Redelivered: 18-SEP-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/*****************************************
* Get_Extractions_from_File declarations *
*****************************************/
void       Get_Extractions_from_File( Plate_List, Jukebox_Path_Table, Options )
PLATE_LIST Plate_List;
LOOK_TAB   Jukebox_Path_Table;
OPTIONS    Options   ;
{
   EXTRACTION Extraction          ; /* Each extraction  read in from file    */
   IS_LIST    Image_Section_List  ; /* List image sections for extractions   */
   FIELD_LIST Field_List          ; /* List of all the fields                */
   HDR_LIST   Header_List         ; /* Header for all plates extracting from */
   X_LIST     Extraction_List     ; /* All extractions from single disc.     */
   X_SET      Extraction_Set      ; /* Set of all extractions                */

/**************************************************
* Get_Extractions_from_File begin executable code *
**************************************************/
   Field_List         = Create_Empty_Field_List        ();
   Image_Section_List = Create_Empty_Image_Section_List();
   Header_List        = Create_Empty_Header_List       ();
   Extraction_List    = Create_Empty_Extraction_List   ();
   Extraction_Set     = Create_Empty_Extraction_Set    ();


/* *** Get_Extractions_from_File ***                                       2 */


   Extraction = Read_Extraction_Info( Options->User_Input_File ,
                                      Plate_List               ,
                                      Options->Header_Directory,
                                      Options->Header_Extension,
                                      Field_List               ,
                                      Image_Section_List       ,
                                      Header_List
                                    );
   while( ( ( int )Extraction != EOF ) &&
                                         ( ( int )Extraction != ( int )NULL ) )
   {
      Add_Extraction_to_List( Extraction_List, Extraction );
      Extraction = Read_Extraction_Info( Options->User_Input_File,
                                         Plate_List               ,
                                         Options->Header_Directory,
                                         Options->Header_Extension,
                                         Field_List               ,
                                         Image_Section_List       ,
                                         Header_List
                                       );
   }
   if ( ( int )Extraction == ( int )NULL ) {
      fprintf( stdout, "\nError while reading extraction information!\n" );
      fprintf( stdout, "Not all extractions may be performed.\n");
   }

   Insert_List_into_Extraction_Set( Extraction_Set, Extraction_List );
   if ( Extraction_Set->Top != NULL )
      Extract_Images_from_Set
	    ( Extraction_Set, Header_List, Jukebox_Path_Table, Options );
   else
      fprintf( stderr, "\nNo images found to extract.\n" );

   Delete_Field_List_Data        ( Field_List         );
   Delete_Image_Section_List_Data( Image_Section_List );
   Delete_Header_List_Data       ( Header_List        );
   Delete_Extraction_Set_Lists   ( Extraction_Set     );

   return;
}
/************************************************
* Get_Extractions_from_File end executable code *
************************************************/

/******************************************************************************
* Module:                                                                     
*    Get_GetImage_Options                                                     
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Gathers options and parameters from the command line.                    
*                                                                             
* Return Value:                                                               
*    OPTIONS: Pointer to the structure containing options obtained from the   
*             command line.                                                   
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       argc                                                                  
*          int, Value                                                         
*             Standard main argument indicating number of words on the        
*              command line.                                                  
*       argv                                                                  
*          char*, Reference                                                   
*             Standard main argument of an array of pointers to the words on  
*             the command line.                                               
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 31-MAY-94 by J. Doggett,                                    
*       Add functionality to override on the command line the CD-ROM mount    
*       specified with the DSS_MTPT environment variable.                     
*    Redelivered: 30-NOV-94 by J. Doggett,                                    
*       GetImage Version 1.1.                                                 
*                                                                             
*    Updated    : 24-JAN-95 by J. Doggett,                                    
*       Added -k option for instructing GetImage to access CD-ROMs from a     
*          jukebox, and added translation of the jukebox environment variables.
*    Redelivered: 18-AUG-95 by J. Doggett,                                    
*       GetImage Version 1.2.                                                 
*                                                                             
*    Updated    : 04-JAN-96 by J. Doggett,                                    
*       Added #ifdef on VMS_NFS_JUKEBOX to force jukebox option when          
*          compiling for jukebox NFS mounted on an OpenVMS machine.           
*       Added option to set plate choice flag.                                
*       Added singlePlate and singlePlateID options to extract from only a    
*          single specified plate.  It uses the -p command line option with   
*          the plate choice option.                                           
*       Added +imagename option and checks for options beginning with "+".    
*    Redelivered: 01-AUG-96 by J. Doggett,                                    
*       GetImage Version 1.3.                                                 
*                                                                             
*    Updated    : 17-SEP-96 by J. Doggett,                                    
******************************************************************************/

/* *** Get_GetImage_Options ***                                            2 */


/******************************************************************************
*       Replaced -k option with +jukebox option which has optional argument
*          specifying the name of the file with paths to discs in a jukebox.
*       Removed translation of old "JUKEBOX" environment variables.
*    Updated    : 21-NOV-96 by J. Doggett,
*       Added default value of TRUE for extractCalibration option.
*       Added acceptance of "nocal" and "cal" options.
*    Updated    : 17-DEC-96 by J. Doggett,                                    
*       Removed use of CDROM_Label_Prefix as that field is not longer used.
*    Updated    : 24-FEB-97 by J. Doggett,                                    
*       Patterned the +jukebox option after the -d option for use with
*       an environment variable.
*    Redelivered: 17-DEC-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/************************************
* Get_GetImage_Options declarations *
************************************/
OPTIONS Get_GetImage_Options( argc, argv )
int     argc   ;
char   *argv[] ;
{
   OPTIONS Options         ; /* Option obtained from the command line */
   int      Argument_N     ; /* For index for looping through command line */
                             /*    arguments.                              */
   char     Argument[ 81 ] ; /* Current argument we're processing          */
   char    *pCurrent_Option; /* For index for looping through a set of     */
                             /*    options grouped together on the coomand */
   int      sscanfResult   ; /* To test the result of a call to sscanf.    */

/*********************************************
* Get_GetImage_Options begin executable code *
*********************************************/
   Options = ( OPTIONS )Allocate_Memory( sizeof( *Options ) );

/*
 * First, we initialize to defaults
 */
   Options->Command_String        =
                          ( STRING )Allocate_Memory( strlen( argv[ 0 ] ) + 1 );
   strcpy( Options->Command_String, argv[ 0 ] );
   Options->Command_String        = NULL                 ;
   Options->Equinox               = J2000                ;
   Options->User_Input            = STDIN_FIELDS         ;
   Options->User_Input_File_Name  = NULL                 ;
   Options->User_Input_File       = NULL                 ;
   Options->User_Output           = STDOUT_IMAGE_SECTIONS;
   Options->User_Output_File_Name = NULL                 ;
   Options->User_Output_File      = NULL                 ;
   Options->Extract_All_Images    = FALSE                ;
   Options->Extract_Images        = TRUE                 ;
   Options->Format                = FITS                 ;
   Options->GEIS_Header_Only      = FALSE                ;
   Options->Batch_Error_Action    = QUERY                ;
   Options->dontUnmountOnExit     = FALSE                ;
   Options->jukeBox               = FALSE                ;
   Options->jukeBoxLookupFile     = NULL                 ;
   Options->Header_Directory      = NULL                 ;
   Options->Header_Extension      = NULL                 ;
   Options->Plate_List_File       = NULL                 ;
   Options->Image_Data_Root_Dir   = NULL                 ;

/* *** Get_GetImage_Options ***                                            3 */


   Options->Mount_Command         = NULL                 ;
   Options->CDROM_Device          = NULL                 ;
   Options->CDROM_Mount_Point     = NULL                 ;
   Options->plateChoiceFlag       = FURTHEST_FROM_EDGE   ;
   Options->singlePlate           = FALSE                ;
   Options->extractCalibrations   = TRUE                 ;
   strcpy(Options->singlePlateID, "");
   strcpy(Options->outputImageName, "");

   for ( Argument_N = 1; Argument_N < argc; Argument_N++ )
   { /* Loop through the arguments on the command line */

/* Get options with arguments
 */
      strcpy( Argument, argv[ Argument_N ] );
      Convert_String_to_Lower_Case( Argument );

/* If -d is present, then set CD-ROM device name
 */
      if ( strcmp( Argument, "-d" ) == 0 )
      {
         if ( ( Argument_N + 1               == argc )
                                    ||
              ( *( argv[ Argument_N + 1 ] ) == '-'   )
                                    ||
              ( *( argv[ Argument_N + 1 ] ) == '+'   ) )
         {
            fprintf(stderr, "Warning: device name missing for -d option.\n");
            fprintf(stderr, "\t -d option ignored.\n");
         }
         else
         {
            strcpy( Argument, argv[ ++Argument_N ] );
            if ( Options->CDROM_Device != NULL )
               free( Options->CDROM_Device );
            Options->CDROM_Device =
                           ( STRING )Allocate_Memory( strlen( Argument ) + 1 );
            strcpy( Options->CDROM_Device, Argument );
         }
      }

/* If -i is present, then set user input to fields from stdin or a file
 */
      else if ( strcmp( Argument, "-i" ) == 0 )
      {
         if ( ( Argument_N + 1              == argc )
                                   ||
              ( *( argv[ Argument_N + 1 ] ) == '-'  ) )
         {
            fprintf(stderr, "Warning: file name missing for -i option.\n");
            fprintf(stderr, "\t -i option ignored.\n");
         }
         else
         {
           strcpy( Argument, argv[ ++Argument_N ] );
           Options->User_Input           = FIELD_FILE;
           Options->User_Input_File_Name =
                           ( STRING )Allocate_Memory( strlen( Argument ) + 1 );
           strcpy( Options->User_Input_File_Name, Argument );
         }
      }

/* *** Get_GetImage_Options ***                                            4 */



/* If -m is present, then set mount point for CD-ROM
 */
      else if ( strcmp( Argument, "-m" ) == 0 )
      {
         if ( ( Argument_N + 1               == argc )
                                    ||
              ( *( argv[ Argument_N + 1 ] ) == '-'   )
                                    ||
              ( *( argv[ Argument_N + 1 ] ) == '+'   ) )
         {
            fprintf(stderr, "Warning: mount point missing for -m option.\n");
            fprintf(stderr, "\t -m option ignored.\n");
         }
         else
         {
            strcpy( Argument, argv[ ++Argument_N ] );
            if ( Options->CDROM_Mount_Point != NULL )
               free( Options->CDROM_Mount_Point );

            Options->CDROM_Mount_Point =
                           ( STRING )Allocate_Memory( strlen( Argument ) + 1 );
            strcpy( Options->CDROM_Mount_Point, Argument );
         }
      }

/* If -o is present, then set user output to stdin or a file.
 */
      else if ( strcmp( Argument, "-o" ) == 0 )
      {
         if ( ( Argument_N + 1              == argc )
                                   ||
              ( *( argv[ Argument_N + 1 ] ) == '-'  ) )
         {
            fprintf(stderr, "Warning: file name missing for -o option.\n");
            fprintf(stderr, "\t -o option ignored.\n");
         }
         else
         {
            strcpy( Argument, argv[ ++Argument_N ] );
            Options->User_Output           = IMAGE_SECTION_FILE;
            Options->User_Output_File_Name =
                           ( STRING )Allocate_Memory( strlen( Argument ) + 1 );
            strcpy( Options->User_Output_File_Name, Argument );
         }
      }

/* If -p is present, then set plate choice flag
 */
      else if ( strcmp( Argument, "-p" ) == 0 )
      {
         if ( ( Argument_N + 1              == argc )
                                   ||
              ( *( argv[ Argument_N + 1 ] ) == '-'  ) )
         {
            fprintf(stderr, "Warning: argument missing for -p option.\n");
            fprintf(stderr, "\t Defaulting to =EDGE value.\n");
            Options->plateChoiceFlag = FURTHEST_FROM_EDGE;
         }
         else
         {

/* *** Get_GetImage_Options ***                                            5 */


            strcpy( Argument, argv[ ++Argument_N ] );
            Convert_String_to_Upper_Case( Argument );

            if (Argument[0] == '=')
            {
               if (Argument[1] == 'C')
                  Options->plateChoiceFlag = CLOSEST_TO_CENTER;
               else if (Argument[1] == 'E')
                  Options->plateChoiceFlag = FURTHEST_FROM_EDGE;
               else
               {
                  fprintf(stderr, "Warning: invalid argument for -p option.\n");
                  fprintf(stderr, "\t Defaulting to =EDGE value.\n");
                  Options->plateChoiceFlag = FURTHEST_FROM_EDGE;
               }
            }
            else
            {
               Options->singlePlate   = TRUE;
               strncpy(Options->singlePlateID, Argument, 4);
               Options->singlePlateID[4] = (char)NULL;
            }
         }
      }

/* If -x is present, then set user input to extractions from stdin or a file.
 */
      else if ( strcmp( Argument, "-x" ) == 0 )
      {
         if ( ( Argument_N + 1               == argc )
                                    ||
              ( *( argv[ Argument_N + 1 ] ) == '-'   )
                                    ||
              ( *( argv[ Argument_N + 1 ] ) == '+'   ) )
         {
            fprintf(stderr, "Warning: file name missing for -x option.\n");
            fprintf(stderr, "\t -x option ignored.\n");
         }
         else
         {
            strcpy( Argument, argv[ ++Argument_N ] );
            Options->User_Input           = IMAGE_SECTION_FILE;
            Options->User_Input_File_Name =
                           ( STRING )Allocate_Memory( strlen( Argument ) + 1 );
            strcpy( Options->User_Input_File_Name, Argument );
         }
      }

/* If +imagename present, then set output image names.
 */
      else if ( strcmp( Argument, "+imagename" ) == 0 )
      {
         if ( ( Argument_N + 1               == argc )
                                    ||
              ( *( argv[ Argument_N + 1 ] ) == '-'   )
                                    ||
              ( *( argv[ Argument_N + 1 ] ) == '+'   ) )
         {
            fprintf(stderr, "Warning: file name missing for -x option.\n");
            fprintf(stderr, "\t -x option ignored.\n");
         }

/* *** Get_GetImage_Options ***                                            6 */


         else
         {
            strcpy( Argument, argv[ ++Argument_N ] );
            strcpy( Options->outputImageName, Argument );
         }
      }

/* If +jukebox present, then set lookup file
 */
      else if ( strcmp( Argument, "+jukebox" ) == 0 )
      {
         Options->jukeBox = TRUE;
         if ( ( Argument_N + 1              != argc )
                                    &&
              ( *( argv[ Argument_N + 1 ] ) != '-'   )
                                    &&
              ( *( argv[ Argument_N + 1 ] ) != '+'   ) )
         {
            strcpy( Argument, argv[ ++Argument_N ] );
            if (Options->jukeBoxLookupFile != NULL)
	       free(Options->jukeBoxLookupFile);
	    Options->jukeBoxLookupFile = 
 	       (STRING)Allocate_Memory(strlen(Argument) + 1);
            strcpy( Options->jukeBoxLookupFile, Argument );
         }
      }

/* Get + options without arguments. They may NOT be grouped together.
 */
   
/* If +nocal present, set extractCalibrations to FALSE
 */
      else if ( strcmp( Argument, "+nocal" ) == 0 )
         Options->extractCalibrations = FALSE;

/* If +cal present, don't do anything except accept option because +cal is the
 * default.
 */
      else if ( strcmp( Argument, "+cal" ) == 0 )
         ;

/* Get - options without arguments. They may be grouped together.
 */
      else if ( *( argv[ Argument_N ] ) == '-' )
      { /* Check options grouped in a single argument */

         for( pCurrent_Option = argv[ Argument_N ] + 1;
                           *pCurrent_Option != ( int )NULL; pCurrent_Option++ )
         { /* Loop through options in current argument */

            switch( *pCurrent_Option )
            {
               case 'a': Options->Extract_All_Images = TRUE ; break;
               case 'c': Options->Extract_All_Images = FALSE; break;

               case 'b': Options->Equinox = B1950; break;
               case 'j': Options->Equinox = J2000; break;

               case 'e': Options->Extract_Images = TRUE ; break;
               case 'n': Options->Extract_Images = FALSE; break;


/* *** Get_GetImage_Options ***                                            7 */


               case 'f': Options->Format = FITS; break;
               case 'g': Options->Format = HHD ; break;

               case 'u': Options->dontUnmountOnExit = TRUE; break;

               case 'w': Options->GEIS_Header_Only = TRUE; break;

               case 's': Options->Batch_Error_Action = SKIP ; break;
               case 'z': Options->Batch_Error_Action = ABORT; break;
               case 'q': Options->Batch_Error_Action = QUERY; break;
               case 'h': Print_Help_and_Stop();

               default:
                  fprintf( stderr, "Unrecognized option, %c, ignored\n",
                                                            *pCurrent_Option );
            }

         } /* Loop through options in current argument */

      } /* Check options grouped in a single argument */

      else
         fprintf( stderr, "Extraneous argument \"%s\" ignored.\n",
                                                          argv[ Argument_N ] );
   } /* Loop through the arguments on the command line */

/*
 * Now get options from configuration macros and environment variables
 */
#ifdef __VMS_NFS_JUKEBOX__
   Options->jukeBox = TRUE;
#endif

   Options->Header_Directory     =
                        Translate_Environment_Variable( HEADER_DIRECTORY     );

   Options->Header_Extension     =
                        Translate_Environment_Variable( HEADER_EXTENSION     );

   Options->Plate_List_File      =
                        Make_Plate_List_File_Name( Options->Header_Directory );

   Options->Mount_Command        =
                        Translate_Environment_Variable( MOUNT_COMMAND        );

   Options->Dismount_Command     =
                        Translate_Environment_Variable( DISMOUNT_COMMAND     );

   if (Options->CDROM_Device == NULL)
      Options->CDROM_Device      =
                        Translate_Environment_Variable( CDROM_DEVICE         );

   if (Options->CDROM_Mount_Point == NULL)
      Options->CDROM_Mount_Point =
                        Translate_Environment_Variable( CDROM_MOUNT_POINT    );

   Options->jukeBoxLookupFile =
      Make_Jukebox_Path_File_Name
         (Options->jukeBoxLookupFile, Options->Header_Directory);

#if defined __VMS

/* *** Get_GetImage_Options ***                                            8 */

   Options->Image_Data_Root_Dir =
         Build_Full_VMS_File_Name (Options->CDROM_Device, "000000",  "" );

#elif defined __UNIX
   Options->Image_Data_Root_Dir =
         Build_Full_UNIX_File_Name(Options->CDROM_Mount_Point, "", "");

#else
   fprintf(stderr, "Unknown file system error in Get_GetImage_Options");
   exit(EXIT_FAILURE);

#endif

   return Options;
}
/*******************************************
* Get_GetImage_Options end executable code *
*******************************************/

/******************************************************************************
* Module:                                                                     
*    Make_Candidate_Plate_List                                                
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Get list of plates that may contain a requested field. The distance of   
*    the field from the centers of plates in the input plate list are         
*    computed. Those plate centers that are within SEARCH_RADIUS_DEGREES of   
*    the field, then those plates are added to the candidate plate list.      
*                                                                             
* Return Value:                                                               
*    PLATE_LIST: Linked list of candidate plates.                             
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Plate_List                                                            
*          PLATE_LIST, Reference                                              
*             List of plates to check for the requested field.                
*       Field                                                                 
*          FIELD, Reference                                                   
*             The field for which we're looking for candidate plates.         
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-Jun-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/************************************
* Make_Candidate_Plate declarations *
************************************/
PLATE_LIST Make_Candidate_Plate_List( Plate_List, Field )
PLATE_LIST Plate_List;
FIELD      Field     ;
{
   PL_NODE    Plate_List_Node     ; /* Node in the input plate list          */
   PLATE_LIST Candidate_Plate_List; /* List of candiate plates which may     *
                                     *    contain  the requested field       */
   PLATE      Candidate_Plate     ; /* Plate info for a candidate plate.     */
   double     distance            ; /* Distance of the field from a plate    *
                                     *    center.                            */
   double     Search_Radius_Radians = SEARCH_RADIUS_DEGREES*PI/180.;
                                    /* Minimum distance in radians of a      *
                                     *    plate center from the requested    *
                                     *    field for it to be a candidate     *
                                     *       plate.                          */

/**************************************************
* Make_Candidate_Plate_List begin executable code *
**************************************************/
/*
 * First initialize candidate plate list.
 */

/* *** Make_Candidate_Plate ***                                            2 */


   Candidate_Plate_List = Create_Empty_Plate_List();

/* Loop through input plate list, picking out those whose center is within
 * SEARCH_RADIUS_DEGREES of the field center
 */

   Plate_List_Node = Plate_List->Top;
   while ( Plate_List_Node != NULL )
   {
      Candidate_Plate = ( PLATE )( Plate_List_Node->Data );

      distance = Great_Circle_Separation
            ( Field->Center_RA          , Field->Center_Dec          ,
              Candidate_Plate->Center_RA, Candidate_Plate->Center_Dec  );

      if ( distance <= Search_Radius_Radians )
         Add_Plate_to_List( Candidate_Plate_List, Candidate_Plate );

      Plate_List_Node = Plate_List_Node->Next;
   }
   return Candidate_Plate_List;
}
/************************************************
* Make_Candidate_Plate_List end executable code *
************************************************/

/******************************************************************************
* Module:                                                                     
*    Make_GetImage_Image_File_Root                                            
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Creates root name for image file for GetImage. It has the form ffffpppp  
*    where ffff is the name of the field, and pppp is the name of the plate   
*    for the image. The field and plate are taken from the extraction         
*    information for the image.                                               
*                                                                             
* Return Value:                                                               
*    STRING: The created root name.                                           
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Extraction                                                            
*          EXTRACTION, Reference                                              
*             The extraction with information for creating the root name.     
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 16-DEC-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/*********************************************
* Make_GetImage_Image_File_Root declarations *
*********************************************/
STRING     Make_GetImage_Image_File_Root( Extraction )
EXTRACTION Extraction;
{
   STRING imageFileRoot   ;
   int    imageFileRootLen;
   int    fieldNameLen    ;
   int    plateNameLen    ;

/******************************************************
* Make_GetImage_Image_File_Root begin executable code *
******************************************************/
   fieldNameLen     = strlen(Extraction->Field->Name);
   plateNameLen     = strlen(Extraction->Plate->Id  );
   imageFileRootLen = fieldNameLen + plateNameLen    ;

   imageFileRoot = (STRING)Allocate_Memory(imageFileRootLen + 1 );

   strcpy(imageFileRoot, Extraction->Field->Name);
   strcat(imageFileRoot, Extraction->Plate->Id  );
   Convert_String_to_Lower_Case( &imageFileRoot[ fieldNameLen ]);

   return imageFileRoot;
}



/* *** Make_GetImage_Image_File_Root ***                                   2 */


/****************************************************
* Make_GetImage_Image_File_Root end executable code *
****************************************************/

/******************************************************************************
* Module:                                                                     
*    Make_Jukebox_Path_File_Name                                            
*                                                                             
* File:                                                                       
*    getimage.c
*                                                                             
* Description:                                                                
*    Create full file name for jukebox path list from given directory, and the
*    symbolic constants JUKEBOX_LOOKUP_FILE or DEFAULT_JUKEBOX_LOOKUP_FILE.
*                                                                             
* Return Value:                                                               
*    STRING: Name of jukebox file.
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Input_File_Name
*          STRING, Reference
*             Jukebox path file name without directory path.
*       Directory                                                             
*          STRING, Reference                                                  
*             Name of directory where plate list file resides.                
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 26-FEB-97 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 26-FEB-97 by J. Doggett,                                    
*       GetImage Version 2.0x.
******************************************************************************/
/*******************************************
* Make_Jukebox_Path_File_Name declarations *
*******************************************/
STRING Make_Jukebox_Path_File_Name(Input_File_Name, Directory)
STRING Input_File_Name;
STRING Directory      ;
{
   STRING Jukebox_Path_List_File_Name     ; /* Place for just the file name */
   STRING Full_Jukebox_Path_List_File_Name; /* File name with path          */

/****************************************************
* Make_Jukebox_Path_File_Name begin executable code *
****************************************************/

   if ((Input_File_Name == (STRING)NULL ) ||
       (strcmp(Input_File_Name, "") == 0)   )
   {
      Jukebox_Path_List_File_Name =
         Translate_Environment_Variable( JUKEBOX_LOOKUP_FILE );

      if ( strcmp(Jukebox_Path_List_File_Name, "") == 0)
      {
         Jukebox_Path_List_File_Name = 
	    (STRING)Allocate_Memory(strlen(DEFAULT_JUKEBOX_LOOKUP_FILE) + 1);

	 strcpy(Jukebox_Path_List_File_Name, DEFAULT_JUKEBOX_LOOKUP_FILE);
      }

/* *** Make_Jukebox_Path_File_Name ***                                     2 */


   }
   else
   {
      Jukebox_Path_List_File_Name = 
         (STRING)Allocate_Memory(strlen(Input_File_Name) + 1);

      strcpy(Jukebox_Path_List_File_Name, Input_File_Name);
   }

   
   if (!Set_FileNames_Case(Directory, Jukebox_Path_List_File_Name))
   {
      fprintf(stderr,"\nCould not find copyright file in header directory!\n");
      fprintf(stderr, "Aborting GetImage!!!\n\n");
      exit(EXIT_FAILURE);
   }

   Full_Jukebox_Path_List_File_Name =
         Build_Full_File_Name( Directory, "", Jukebox_Path_List_File_Name );

   free(Jukebox_Path_List_File_Name);

   Jukebox_Path_List_File_Name =
         Set_FileNames_Version(Directory, Full_Jukebox_Path_List_File_Name);

   if (Jukebox_Path_List_File_Name != Full_Jukebox_Path_List_File_Name)
      if (Jukebox_Path_List_File_Name != NULL)
         free(Full_Jukebox_Path_List_File_Name);
      else {
         fprintf(stderr,
               "\nCould not find copyright file in header directory!\n");
         fprintf(stderr, "Aborting GetImage!!!\n\n");
         exit(EXIT_FAILURE);
      }

   Full_Jukebox_Path_List_File_Name = Jukebox_Path_List_File_Name;

   if (access(Full_Jukebox_Path_List_File_Name, 0) != 0 )
   {
      Jukebox_Path_List_File_Name = 
         (STRING)Allocate_Memory(strlen(DEFAULT_JUKEBOX_LOOKUP_FILE) + 1);

      strcpy(Jukebox_Path_List_File_Name, DEFAULT_JUKEBOX_LOOKUP_FILE);

      if (!Set_FileNames_Case(Directory, Jukebox_Path_List_File_Name))
      {
         fprintf(stderr,
               "\nCould not find copyright file in header directory!\n");
         fprintf(stderr, "Aborting GetImage!!!\n\n");
         exit(EXIT_FAILURE);
      }

      Full_Jukebox_Path_List_File_Name =
         Build_Full_File_Name( Directory, "", Jukebox_Path_List_File_Name );

      free(Jukebox_Path_List_File_Name);

      Jukebox_Path_List_File_Name =
            Set_FileNames_Version(Directory, Full_Jukebox_Path_List_File_Name);

      if (Jukebox_Path_List_File_Name != Full_Jukebox_Path_List_File_Name)

/* *** Make_Jukebox_Path_File_Name ***                                     3 */


         if (Jukebox_Path_List_File_Name != NULL)
            free(Full_Jukebox_Path_List_File_Name);
         else {
            fprintf(stderr,
                  "\nCould not find copyright file in header directory!\n");
            fprintf(stderr, "Aborting GetImage!!!\n\n");
            exit(EXIT_FAILURE);
         }

      Full_Jukebox_Path_List_File_Name = Jukebox_Path_List_File_Name;
   }

   return Full_Jukebox_Path_List_File_Name;
}
/**************************************************
* Make_Jukebox_Path_File_Name end executable code *
**************************************************/

/******************************************************************************
* Module:                                                                     
*    mountCDROM                                                               
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Mounts a CD-ROM disc. The command string is constructed given the base   
*    system command, the device name for the CD-ROM drive, the volume label,  
*    and the mount point in the file system (for UNIX) depending on the       
*    operating system. The command string is then passed to the operating     
*    system with the "system" function.                                       
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       BaseCommand                                                           
*          STRING, Reference                                                  
*             Basic command with any options for mounting a CD-ROM disc.      
*       Device                                                                
*          STRING, Reference                                                  
*             Device name of the CD-ROM drive.                                
*       volumeLabel                                                           
*          STRING, Reference                                                  
*             Volume label for the CD-ROM disc to be mounted.                 
*       mountPoint                                                            
*          STRING, Reference                                                  
*             Point in the file system for UNIX systems where the CD-ROM      
*             disc is to be mounted.                                          
*    Input/Output Arguments:                                                  
*          none                                                               
*    Output Arguments:                                                        
*          none                                                               
*                                                                             
* History:                                                                    
*    Created    : 10-JAN-94 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 17-OCT-94 by J. Doggett,                                    
*       Modified to facilitate ss_mount program for use in managing the       
*       mounting of CD-ROMs on UNIX systems.                                  
*    Redelivered: 30-NOV-94 by J. Doggett,                                    
*       GetImage Version 1.1.                                                 
*                                                                             
*    Updated    : 01-JUN-95 by J. Doggett,                                    
*       Replaced non-ANSI standard function strdup with malloc and strcpy.    
*    Redelivered: 18-AUG-95 by J. Doggett,                                    
*       GetImage Version 1.2.                                                 
******************************************************************************/
/**************************
* mountCDROM declarations *
**************************/
#ifdef USE_SSMOUNT
void mountCDROM(Options, volumeLabel)
OPTIONS Options;

/* *** mountCDROM ***                                                      2 */


STRING volumeLabel;
#else
void mountCDROM(BaseCommand, Device, volumeLabel, mountPoint)
STRING BaseCommand;
STRING Device     ;
STRING volumeLabel;
STRING mountPoint ;
#endif
{
   STRING mountCommand; /* String with mount command to send to the */
                        /* system.                                  */

/***********************************
* mountCDROM begin executable code *
***********************************/
#ifdef USE_SSMOUNT
        FILE *fp;
        char buf[1024];
        mountCommand = createUNIX_CDROMmountCommand(Options->Mount_Command,
                Options->CDROM_Device, Options->CDROM_Mount_Point, volumeLabel);
#else
#if defined __VMS
   mountCommand =
         createVMS_CDROMmountCommand(BaseCommand, Device, volumeLabel);
   printf("Mounting %s on %s...", volumeLabel, Device);
#elif defined __UNIX
   mountCommand =
         createUNIX_CDROMmountCommand(BaseCommand, Device, mountPoint);
   printf("Mounting %s on %s at %s...", volumeLabel, Device, mountPoint);
#else
   fprintf(stderr,"Unknown file system error in mountCDROM");
   exit(EXIT_FAILURE);
#endif
#endif

   fflush(stdout);

#ifdef USE_SSMOUNT
   fp = popen(mountCommand, "r");
   if(fgets(buf, sizeof(buf)-1, fp) == NULL)
        return;

   buf[strlen(buf)-1] = '\0';
   if(Options->CDROM_Mount_Point)
        free(Options->CDROM_Mount_Point);
   Options->CDROM_Mount_Point = (char *)malloc(strlen(buf) + 1);
   strcpy(Options->CDROM_Mount_Point, buf);

   if (Options->Image_Data_Root_Dir)
        free(Options->Image_Data_Root_Dir);

   Options->Image_Data_Root_Dir =
         Build_Full_UNIX_File_Name(Options->CDROM_Mount_Point, "", "");

   if(fgets(buf, sizeof(buf)-1, fp) == NULL)
        return;

   buf[strlen(buf)-1] = '\0';
   if(Options->CDROM_Device)
        free(Options->CDROM_Device);


/* *** mountCDROM ***                                                      3 */


   Options->CDROM_Device = (char *)malloc(strlen(buf) + 1);
   strcpy(Options->CDROM_Device, buf);
#else
   system(mountCommand);
#endif

   printf("\n");
   free(mountCommand);
}
/*********************************
* mountCDROM end executable code *
*********************************/

/******************************************************************************
* Module:                                                                     
*    Open_User_Input                                                          
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Opens the file where information for data to extract will be read. The   
*    input file may contain field data, ie, right ascensions and              
*    declinations, or it may contains image section data, ie, x-y pixel       
*    positions on plates.                                                     
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Options                                                               
*          OPTIONS, Reference                                                 
*             Structure with options obtained from the command line.          
*    Input/Output Arguments:                                                  
*          none                                                               
*    Output Arguments:                                                        
*          none                                                               
*                                                                             
* History:                                                                    
*    Created    : 17-AUG-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/*******************************
* Open_User_Input declarations *
*******************************/
void Open_User_Input( Options )
OPTIONS Options;
{
/****************************************
* Open_User_Input begin executable code *
****************************************/
   if ( ( Options->User_Input == FIELD_FILE         )
                            ||
        ( Options->User_Input == IMAGE_SECTION_FILE ) )
   {
      Options->User_Input_File = fopen( Options->User_Input_File_Name, "r" );
      if ( Options->User_Input_File == NULL )
      {
         Print_Usage();
         fprintf( stderr, "Could not open input file, %s.\n\n",
                                               Options->User_Input_File_Name );
         exit( EXIT_FAILURE );
      }
   }
   else
      Options->User_Input_File = stdin;

   return;

}

/* *** Open_User_Input ***                                                 2 */


/**************************************
* Open_User_Input end executable code *
**************************************/

/******************************************************************************
* Module:                                                                     
*    Open_User_Output                                                         
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Opens the path to where information for each of the extractions will be  
*    written.                                                                 
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Options                                                               
*          OPTIONS, Reference                                                 
*             Structure with options obtained from the command line.          
*    Input/Output Arguments:                                                  
*          none                                                               
*    Output Arguments:                                                        
*          none                                                               
*                                                                             
* History:                                                                    
*    Created    : 17-AUG-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/********************************
* Open_User_Output declarations *
********************************/
void Open_User_Output( Options )
OPTIONS Options;
{
/*****************************************
* Open_User_Output begin executable code *
*****************************************/
   if ( Options->User_Output == IMAGE_SECTION_FILE )
   {
      if (access(Options->User_Output_File_Name, 0) == 0 )
      {
#if defined __VMS
         fprintf
            (stderr, "A new version of %s will be created.\n",
             Options->User_Output_File_Name
            );

#elif defined __UNIX
         fprintf(stderr, "Error opening output file.\n");
         fprintf(stderr, "%s already exists.\n",
               Options->User_Output_File_Name);
         fprintf(stderr, "GetImage aborted.\n\n");
         exit( EXIT_FAILURE );
#else
         fprintf(stderr,"Unknown file system error in Open_User_Output");
         exit(EXIT_FAILURE);

#endif

/* *** Open_User_Output ***                                                2 */


      }

      Options->User_Output_File = fopen( Options->User_Output_File_Name, "w" );

      if ( Options->User_Output_File == NULL )
      {
         Print_Usage();
         fprintf( stderr, "Could not open output file, %s.\n\n",
                                              Options->User_Output_File_Name );
         exit( EXIT_FAILURE );
      }
   }
   else
      Options->User_Output_File = stdout;

   return;
}
/***************************************
* Open_User_Output end executable code *
***************************************/

/******************************************************************************
* Module:                                                                     
*    Print_Help_and_Stop                                                      
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Prints GetImage usage, and descriptions of available options, then       
*    stops execution.                                                         
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments:                                                                  
*    Input Arguments:                                                         
*       Options                                                               
*          OPTIONS, Reference                                                 
*             Structure with options obtained from the command line.          
*    Input/Output Arguments:                                                  
*          none                                                               
*    Output Arguments:                                                        
*          none                                                               
*                                                                             
* History:                                                                    
*    Created    : 19-NOV-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 01-JUN-94 by J. Doggett,                                    
*       Added description of new -m/-M option to override environment         
*       variable specification of the UNIX mount point for the CD-ROM.        
*    Redelivered: 30-NOV-94 by J. Doggett,                                    
*       GetImage Version 1.1.                                                 
*                                                                             
*    Updated    : 24-JAN-95 by J. Doggett,                                    
*       Added description of new -k/-K option for extracting images from a    
*       CD-ROM jukebox.                                                       
*    Redelivered: 18-AUG-95 by J. Doggett,                                    
*       GetImage Version 1.2.                                                 
*                                                                             
*    Updated    : 27-FEB-96 by J. Doggett,                                    
*       Added description for new -p option.                                  
*       Added description for new +imagename option.                          
*    Redelivered: 01-AUG-96 by J. Doggett,                                    
*       GetImage Version 1.3.                                                 
*                                                                             
*    Updated    : 17-SEP-96 by J. Doggett,                                    
*       Replaced descirption of -k option with description of +jukebox        
*       option.                                                               
*    Redelivered: 17-SEP-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/***********************************
* Print_Help_and_Stop declarations *
***********************************/
void Print_Help_and_Stop()
{
/********************************************
* Print_Help_and_Stop begin executable code *
********************************************/
   Print_Usage();

   printf("\n");
   printf("   -a    %s\n",
                          "Extract images from all plates a field occurs on.");

/* *** Print_Help_and_Stop ***                                             2 */


   printf("   -b    %s\n", "Input field coordinates are equinox B1950.");
   printf("   -c    %s\n",
                "Extract only one image from the choice plate.");
   printf("   -d DEVICE\n         %s\n",
             "Override environment variable for CD-ROM drive with DEVICE.");
   printf("   -e    %s\n", "Peform image extractions.");
   printf("   -f    %s\n", "Write extracted images to FITS files.");
   printf("   -g    %s\n", "Write extracted images to GEIS files (hhh/hhd).");
   printf("   -h    %s\n", "Print this help message");
   printf("   -i FILE\n         %s\n",
             "Read field information from file or stdin if FILE not present.");
   printf("   +imagename FILEROOT\n");
   printf("         %s\n", "Create image file names with root FILEROOT.");
   printf("   +jukebox LOOKUPTABLE\n");
   printf("         %s", "Read data from jukebox with paths to discs in the");
   printf(" %s\n"      , "LOOKUPTABLE");
   printf("   -j    %s\n", "Input field coordinates are equinox J2000.");
   printf("   -m MNTPNT\n         %s\n",
     "Override environment variable for UNIX CD-ROM mount point with MNTPNT.");
   printf("   -n    %s\n", "Don't perform image extractions.");
   printf("   -o FILE\n         %s\n",
         "Save extraction information to file or stdin if FILE not present." );
   printf("   -p [=c[enter], =e[dge], PLATE-ID]\n");
   printf("         %s\n",
        "Criterion for selecting plate from which to extract images.");
   printf("   -q    %s %s\n", "In batch mode, query user for action for",
                                          "when required disc not found." );
   printf("   -s    %s\n",
                       "In batch mode, skip extractions not on current disc.");
   printf("   -u    %s\n",
           "Override unmounting of CD-ROM disc when GetImage exits.");
   printf("   -w    %s\n",
           "Override writing of GEIS image files. Only write header file.");
   printf("   -x FILE\n         %s\n",
           "Read extraction information from file created with -o/-O option.");
   printf("   -z    %s\n",
                  "In batch mode, abort if extractions not on current disc.");

   printf("\nNote that the option flags are case insensitive.\n");

   printf("\nBatch modes are the -i or -x option with a file\n");

   printf("\nThe default command line options are:\n");
   printf("\t-cefijoq\n\n");

   exit(EXIT_SUCCESS);
}
/******************************************
* Print_Help_and_Stop end executable code *
******************************************/

/******************************************************************************
* Module:                                                                     
*    Print_Interactive_Instructions                                           
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Print instruction for how to use GetImage interactively.                 
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Command                                                               
*          STRING, Reference                                                  
*             The name of the command used to invoke getimage.                
*    Input/Output Arguments:                                                  
*          none                                                               
*    Output Arguments:                                                        
*          none                                                               
*                                                                             
* History:                                                                    
*    Created    : 17-AUG-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 28-SEP-95 by J. Doggett,                                    
*       Modified to reflect increase in limit of field name size.             
*    Redelivered: 01-AUG-96 by J. Doggett,                                    
*       GetImage Version 1.3.                                                 
******************************************************************************/
/**********************************************
* Print_Interactive_Instructions declarations *
***********************************************/
void    Print_Interactive_Instructions( Options )
OPTIONS Options;
{
/*******************************************************
* Print_Interactive_Instructions begin executable code *
*******************************************************/
   printf( "\nFor each field, enter:\n" );
   printf( "\t* Field name of %d characters or less valid for a filename,\n",
               FIELD_NAME_MAX_LENGTH);
   if ( Options->Equinox == J2000 )
      printf( "\n\t* Center coordinates (%s):\n", "J2000" );
   else
      printf( "\n\t* Center coordinates (%s):\n", "B1950" );

   printf("\t\tRight ascension:  hh mm ss.sss in free format,\n");
   printf("\t\tDeclination    : Sdd mm ss.ss  in free format,\n");
   printf("\t\t                  (where S is an optional sign),\n");
   printf("\n\t* Width and Height of field in arcminutes in free format.\n");
   printf("\n\t\teg, \"Obj1  1 2 3.4  -0 1 2.3  4.5 6.7\"\n");
   printf( "\nOR\n" );
   printf( "\t* A simbad target name within square bracket,\n" );
   printf( "\t* Field name of maximum 8 characters valid for a filename,\n" );
   printf("\n\t* Width and Height of field in arcminutes in free format\n");
   printf("\n\n\t\t [target] Obj1  4.5 6.7\n");
   printf(
       "\nTerminate the input list with EOF signal (^d UNIX; ^z OpenVMS).\n" );

   return;

/* *** Print_Interactive_Instructions ***                                  2 */

}
/*****************************************************
* Print_Interactive_Instructions end executable code *
*****************************************************/

/******************************************************************************
* Module:                                                                     
*    Print_Usage                                                              
*                                                                             
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Prints the usage of the getimage command.                                
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*          none                                                               
*    Input/Output Arguments:                                                  
*          none                                                               
*    Output Arguments:                                                        
*          none                                                               
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 01-JUN-94 by J. Doggett,                                    
*       Added new -m/-M option to override environment variable specification 
*       of the UNIX mount point for the CD-ROM.                               
*    Redelivered: 30-NOV-94 by J. Doggett,                                    
*       GetImage Version 1.1.                                                 
*                                                                             
*    Updated    : 24-JAN-95 by J. Doggett,                                    
*       Added new -k/-K option for extracting images from a  CD-ROM jukebox.  
*    Redelivered: 18-AUG-95 by J. Doggett,                                    
*       GetImage Version 1.2.                                                 
*                                                                             
*    Updated    : 27-FEB-96 by J. Doggett,                                    
*       Added description for new -p option.                                  
*       Added description for new +imagename option.                          
*    Redelivered: 01-AUG-96 by J. Doggett,                                    
*       GetImage Version 1.3.                                                 
*                                                                             
*    Updated    : 17-SEP-96 by J. Doggett,                                    
*       Replaced -k option with +jukebox option.                              
*    Redelivered: 17-SEP-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/***************************
* Print_Usage declarations *
***************************/
void Print_Usage()
{
/************************************
* Print_Usage begin executable code *
************************************/
   printf( "\nUsage: %s %s %s\n", "getimage",
          "[-abcefghjnqsuwz]",                                                 
          "[-d cd-rom_device]" );
   printf( "\t%s %s\n", "[-i input_file_of_fields]",
                                          "[+imagename image_file_name_root]");
   printf( "\t%s %s\n", "[+jukebox lookup_table_file]",
	                                    "[-m cd-rom_mount_point]");
   printf( "\t%s %s\n","[-o output_file_of_extractions]" ,
                        "[-p selection_criterion]"       );
   printf( "\t%s\n","[-x input_file_of_extractions]");

/* *** Print_Usage ***                                                     2 */

   return;

}
/**********************************
* Print_Usage end executable code *
**********************************/

/******************************************************************************
* Module:                                                                     
*    Read_Jukebox_Path_Table                                                   
*                          						      
* File:                                                                       
*    getimage.c                                                               
*                                                                             
* Description:                                                                
*    Reads the lookup table for paths to discs in a jukebox		      
*                                                                             
* Return Value:                                                               
*    LOOK_TAB lookup table containing the paths to the discs as the dependent
*    values and the disc volume labels as the independent values.
*    (See looktab.c).  If the read fails, then (LOOK_TAB)NULL is returned.
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       file_name							      
*          STRING, value  						      
*    Input/Output Arguments:                                                  
*          none                                                               
*    Output Arguments:                                                        
*          none                                                               
*                                                                             
* History:                                                                    
*    Created    : 18-SEP-96 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 27-FEB-97 by J. Doggett,                                    
*       GetImage Version 2.0x.
******************************************************************************/
/***************************************
* Read_Jukebox_Path_Table declarations *
***************************************/
LOOK_TAB Read_Jukebox_Path_Table(filename)
STRING   filename;
{
   FILE     * path_table_file ;
   LOOK_TAB   table           ;
   char       volume_label[81];
   char       path        [81];
   int        scan_result     ;
   int        Temp_Char       ;

/************************************************
* Read_Jukebox_Path_Table begin executable code *
************************************************/
   if (filename == (STRING)NULL)
      return (LOOK_TAB)NULL;

   printf( "Reading jukebox path list: %s...\n", filename );

   path_table_file = fopen(filename, "r");
      if (path_table_file == (FILE *)NULL)
      {
         fprintf
               (stderr, "Could not open jukebox path list file %s\n", filename);
	 return (LOOK_TAB)NULL;
      }   

   table = Create_LookupTbl();

/* *** Read_Jukebox_Path_Table ***                                          2 */


      if (table == (LOOK_TAB)NULL)
      {
         fprintf
               (stderr, "Could not create jukebox path lookup table.\n");
	 fclose(path_table_file);
	 return (LOOK_TAB)NULL;
      }   
      
  
   scan_result = fscanf(path_table_file, "%s %s", volume_label, path);
   while (scan_result == 2)
   {
      if (volume_label[0] != '#')
      {
	 Convert_String_to_Upper_Case(volume_label);
	 if (!Add_LookupTbl_Entry_to_Tbl(table, volume_label, path))
	 {
	    fprintf(stderr, "Error adding entry to jukebox path lookup table.\n");
	    Del_LookupTbl(table);
	    fclose(path_table_file);
	    return (LOOK_TAB)NULL;
	 }
      }
      else
	while (((Temp_Char=getc(path_table_file)) != '\n') &&
	       (Temp_Char                         != EOF )   )
	   ;

      scan_result = fscanf(path_table_file, "%s %s", volume_label, path);
   }
   if (scan_result != EOF)
   {
      fprintf(stderr, "Error reading jukebox path list file %s\n", filename);
      Del_LookupTbl(table);
      fclose(path_table_file);
      return (LOOK_TAB)NULL;
   }

   fclose(path_table_file);
   return table;
}
/**********************************************
* Read_Jukebox_Path_Table end executable code *
**********************************************/
